@@ -1,85 +1,49 @@
-# Copyright (C) 2009, 2010, 2011, 2012, 2013
-# David Aguilar <davvid@gmail.com>
 """Provides the main() routine and ColaApplication"""
 from __future__ import division, absolute_import, unicode_literals
-
 import argparse
 import os
-import shutil
 import signal
 import sys
 
-# Make homebrew work by default
-if sys.platform == 'darwin':
-    from distutils import sysconfig
-    python_version = sysconfig.get_python_version()
-    homebrew_mods = '/usr/local/lib/python%s/site-packages' % python_version
-    if os.path.isdir(homebrew_mods):
-        sys.path.append(homebrew_mods)
-
-
-errmsg = """Sorry, you do not seem to have PyQt4 installed.
-Please install it before using git-cola.
-e.g.: sudo apt-get install python-qt4
+__copyright__ = """
+Copyright (C) 2009-2016 David Aguilar and contributors
 """
 
-# /usr/include/sysexits.h
-#define EX_OK           0   /* successful termination */
-#define EX_USAGE        64  /* command line usage error */
-#define EX_NOINPUT      66  /* cannot open input */
-#define EX_UNAVAILABLE  69  /* service unavailable */
-EX_OK = 0
-EX_USAGE = 64
-EX_NOINPUT = 66
-EX_UNAVAILABLE = 69
-
-
+from . import core
 try:
-    import sip
+    from qtpy import QtCore
 except ImportError:
-    sys.stderr.write(errmsg)
-    sys.exit(EX_UNAVAILABLE)
+    errmsg = """
+Sorry, you do not seem to have PyQt5, Pyside, or PyQt4 installed.
+Please install it before using git-cola, e.g.:
+    $ sudo apt-get install python-qt4
+"""
+    core.error(errmsg)
 
-sip.setapi('QString', 1)
-sip.setapi('QDate', 1)
-sip.setapi('QDateTime', 1)
-sip.setapi('QTextStream', 1)
-sip.setapi('QTime', 1)
-sip.setapi('QUrl', 1)
-sip.setapi('QVariant', 1)
-
-try:
-    from PyQt4 import QtCore
-except ImportError:
-    sys.stderr.write(errmsg)
-    sys.exit(EX_UNAVAILABLE)
-
-from PyQt4 import QtGui
-from PyQt4.QtCore import Qt
-from PyQt4.QtCore import SIGNAL
+from qtpy import QtGui
+from qtpy import QtWidgets
 
 # Import cola modules
-from cola import cmds
-from cola import core
-from cola import compat
-from cola import git
-from cola import gitcfg
-from cola import icons
-from cola import inotify
-from cola import i18n
-from cola import qtcompat
-from cola import qtutils
-from cola import resources
-from cola import utils
-from cola import version
-from cola.compat import ustr
-from cola.decorators import memoize
-from cola.i18n import N_
-from cola.interaction import Interaction
-from cola.models import main
-from cola.widgets import cfgactions
-from cola.widgets import startup
-from cola.settings import Session
+from .decorators import memoize
+from .i18n import N_
+from .interaction import Interaction
+from .models import main
+from .widgets import cfgactions
+from .widgets import defs
+from .widgets import startup
+from .settings import Session
+from . import cmds
+from . import core
+from . import compat
+from . import fsmonitor
+from . import git
+from . import gitcfg
+from . import icons
+from . import i18n
+from . import qtcompat
+from . import qtutils
+from . import resources
+from . import version
 
 
 def setup_environment():
@@ -87,7 +51,7 @@ def setup_environment():
     signal.signal(signal.SIGINT, signal.SIG_DFL)
 
     # Session management wants an absolute path when restarting
-    sys.argv[0] = sys_argv0 = core.abspath(sys.argv[0])
+    sys.argv[0] = sys_argv0 = os.path.abspath(sys.argv[0])
 
     # Spoof an X11 display for SSH
     os.environ.setdefault('DISPLAY', ':0')
@@ -100,7 +64,7 @@ def setup_environment():
 
     # Setup the path so that git finds us when we run 'git cola'
     path_entries = core.getenv('PATH', '').split(os.pathsep)
-    bindir = os.path.dirname(sys_argv0)
+    bindir = core.decode(os.path.dirname(sys_argv0))
     path_entries.append(bindir)
     path = os.pathsep.join(path_entries)
     compat.setenv('PATH', path)
@@ -156,6 +120,24 @@ def setup_environment():
     compat.setenv('GIT_MERGE_AUTOEDIT', 'no')
 
 
+def get_icon_themes():
+    """Return the default icon theme names"""
+    themes = []
+
+    icon_themes_env = core.getenv('GIT_COLA_ICON_THEME')
+    if icon_themes_env:
+        themes.extend([x for x in icon_themes_env.split(':') if x])
+
+    icon_themes_cfg = gitcfg.current().get_all('cola.icontheme')
+    if icon_themes_cfg:
+        themes.extend(icon_themes_cfg)
+
+    if not themes:
+        themes.append('light')
+
+    return themes
+
+
 # style note: we use camelCase here since we're masquerading a Qt class
 class ColaApplication(object):
     """The main cola application
@@ -163,25 +145,91 @@ class ColaApplication(object):
     ColaApplication handles i18n of user-visible data
     """
 
-    def __init__(self, argv, locale=None, gui=True):
+    def __init__(self, argv, locale=None, gui=True, icon_themes=None):
         cfgactions.install()
         i18n.install(locale)
         qtcompat.install()
         qtutils.install()
-        icons.install()
+        icons.install(icon_themes or get_icon_themes())
 
-        self.notifier = QtCore.QObject()
-        self.notifier.connect(self.notifier, SIGNAL('update_files()'),
-                              self._update_files, Qt.QueuedConnection)
-        # Call _update_files when inotify detects changes
-        inotify.observer(self._update_files_notifier)
+        fsmonitor.current().files_changed.connect(self._update_files)
 
         if gui:
             self._app = current(tuple(argv))
             self._app.setWindowIcon(icons.cola())
+            self._install_style()
         else:
             self._app = QtCore.QCoreApplication(argv)
 
+    def _install_style(self):
+        palette = self._app.palette()
+        window = palette.color(QtGui.QPalette.Window)
+        highlight = palette.color(QtGui.QPalette.Highlight)
+        shadow = palette.color(QtGui.QPalette.Shadow)
+        base = palette.color(QtGui.QPalette.Base)
+
+        window_rgb = qtutils.rgb_css(window)
+        highlight_rgb = qtutils.rgb_css(highlight)
+        shadow_rgb = qtutils.rgb_css(shadow)
+        base_rgb = qtutils.rgb_css(base)
+
+        self._app.setStyleSheet("""
+            QCheckBox::indicator {
+                width: %(checkbox_size)spx;
+                height: %(checkbox_size)spx;
+            }
+            QCheckBox::indicator::unchecked {
+                border: %(checkbox_border)spx solid %(shadow_rgb)s;
+                background: %(base_rgb)s;
+            }
+            QCheckBox::indicator::checked {
+                image: url(%(checkbox_icon)s);
+                border: %(checkbox_border)spx solid %(shadow_rgb)s;
+                background: %(base_rgb)s;
+            }
+
+            QRadioButton::indicator {
+                width: %(radio_size)spx;
+                height: %(radio_size)spx;
+            }
+            QRadioButton::indicator::unchecked {
+                border: %(radio_border)spx solid %(shadow_rgb)s;
+                border-radius: %(radio_radius)spx;
+                background: %(base_rgb)s;
+            }
+            QRadioButton::indicator::checked {
+                image: url(%(radio_icon)s);
+                border: %(radio_border)spx solid %(shadow_rgb)s;
+                border-radius: %(radio_radius)spx;
+                background: %(base_rgb)s;
+            }
+
+            QSplitter::handle:hover {
+                background: %(highlight_rgb)s;
+            }
+
+            QMainWindow::separator {
+                background: %(window_rgb)s;
+                width: %(separator)spx;
+                height: %(separator)spx;
+            }
+            QMainWindow::separator:hover {
+                background: %(highlight_rgb)s;
+            }
+
+            """ % dict(separator=defs.separator,
+                       window_rgb=window_rgb,
+                       highlight_rgb=highlight_rgb,
+                       shadow_rgb=shadow_rgb,
+                       base_rgb=base_rgb,
+                       checkbox_border=defs.border,
+                       checkbox_icon=icons.check_name(),
+                       checkbox_size=defs.checkbox,
+                       radio_border=defs.radio_border,
+                       radio_icon=icons.dot_name(),
+                       radio_radius=defs.checkbox//2,
+                       radio_size=defs.checkbox))
+
     def activeWindow(self):
         """Wrap activeWindow()"""
         return self._app.activeWindow()
@@ -198,37 +246,34 @@ class ColaApplication(object):
             self._app.view = view
 
     def _update_files(self):
-        # Respond to inotify updates
+        # Respond to file system updates
         cmds.do(cmds.Refresh)
 
-    def _update_files_notifier(self):
-        self.notifier.emit(SIGNAL('update_files()'))
-
 
 @memoize
 def current(argv):
     return ColaQApplication(list(argv))
 
 
-class ColaQApplication(QtGui.QApplication):
+class ColaQApplication(QtWidgets.QApplication):
 
     def __init__(self, argv):
-        QtGui.QApplication.__init__(self, argv)
-        self.view = None ## injected by application_start()
+        super(ColaQApplication, self).__init__(argv)
+        self.view = None  # injected by application_start()
 
     def event(self, e):
         if e.type() == QtCore.QEvent.ApplicationActivate:
             cfg = gitcfg.current()
             if cfg.get('cola.refreshonfocus', False):
                 cmds.do(cmds.Refresh)
-        return QtGui.QApplication.event(self, e)
+        return super(ColaQApplication, self).event(e)
 
     def commitData(self, session_mgr):
         """Save session data"""
         if self.view is None:
             return
-        sid = ustr(session_mgr.sessionId())
-        skey = ustr(session_mgr.sessionKey())
+        sid = session_mgr.sessionId()
+        skey = session_mgr.sessionKey()
         session_id = '%s_%s' % (sid, skey)
         session = Session(session_id, repo=core.getcwd())
         self.view.save_state(settings=session)
@@ -238,7 +283,7 @@ def process_args(args):
     if args.version:
         # Accept 'git cola --version' or 'git cola version'
         version.print_version()
-        sys.exit(EX_OK)
+        sys.exit(core.EXIT_SUCCESS)
 
     # Handle session management
     restore_session(args)
@@ -252,11 +297,7 @@ def process_args(args):
         errmsg = N_('fatal: "%s" is not a directory.  '
                     'Please specify a correct --repo <path>.') % repo
         core.stderr(errmsg)
-        sys.exit(EX_USAGE)
-
-    # We do everything relative to the repo root
-    os.chdir(args.repo)
-    return repo
+        sys.exit(core.EXIT_USAGE)
 
 
 def restore_session(args):
@@ -300,24 +341,18 @@ def application_start(context, view):
     runtask = qtutils.RunTask(parent=view)
     init_update_task(view, runtask, context.model)
 
-    # Start the inotify thread
-    inotify.start()
+    # Start the filesystem monitor thread
+    fsmonitor.current().start()
 
-    msg_timer = QtCore.QTimer()
-    msg_timer.setSingleShot(True)
-    msg_timer.connect(msg_timer, SIGNAL('timeout()'), _send_msg)
-    msg_timer.start(0)
+    QtCore.QTimer.singleShot(0, _send_msg)
 
     # Start the event loop
     result = context.app.exec_()
 
     # All done, cleanup
-    inotify.stop()
+    fsmonitor.current().stop()
     QtCore.QThreadPool.globalInstance().waitForDone()
 
-    tmpdir = utils.tmpdir()
-    shutil.rmtree(tmpdir, ignore_errors=True)
-
     return result
 
 
@@ -334,6 +369,11 @@ def add_common_arguments(parser):
     parser.add_argument('--prompt', action='store_true', default=False,
                         help='prompt for a repository')
 
+    # Specify the icon theme
+    parser.add_argument('--icon-theme', metavar='<theme>',
+                        dest='icon_themes', action='append', default=[],
+                        help='specify an icon theme (name or directory)')
+
     # Resume an X Session Management session
     parser.add_argument('-session', metavar='<session>', default=None,
                         help=argparse.SUPPRESS)
@@ -341,22 +381,33 @@ def add_common_arguments(parser):
 
 def new_application(args):
     # Initialize the app
-    return ColaApplication(sys.argv)
+    return ColaApplication(sys.argv, icon_themes=args.icon_themes)
 
 
 def new_model(app, repo, prompt=False, settings=None):
     model = main.model()
-    valid = model.set_worktree(repo) and not prompt
+    valid = False
+    if not prompt:
+        valid = model.set_worktree(repo)
+        if not valid:
+            # We are not currently in a git repository so we need to find one.
+            # Before prompting the user for a repository, check if they've
+            # configured a default repository and attempt to use it.
+            default_repo = gitcfg.current().get('cola.defaultrepo')
+            if default_repo:
+                valid = model.set_worktree(default_repo)
+
     while not valid:
+        # If we've gotten into this loop then that means that neither the
+        # current directory nor the default repository were available.
+        # Prompt the user for a repository.
         startup_dlg = startup.StartupDialog(app.activeWindow(),
                                             settings=settings)
         gitdir = startup_dlg.find_git_repo()
         if not gitdir:
-            sys.exit(EX_NOINPUT)
+            sys.exit(core.EXIT_NOINPUT)
         valid = model.set_worktree(gitdir)
 
-    # Finally, go to the root of the git repo
-    os.chdir(model.git.worktree())
     return model
 
 
@@ -375,9 +426,17 @@ def init_update_task(parent, runtask, model):
 
 
 def _send_msg():
-    if git.GIT_COLA_TRACE == 'trace':
-        msg = 'info: debug mode enabled using GIT_COLA_TRACE=trace'
-        Interaction.log(msg)
+    trace = git.GIT_COLA_TRACE
+    if trace == '2' or trace == 'trace':
+        msg1 = 'info: debug level 2: trace mode enabled'
+        msg2 = 'info: set GIT_COLA_TRACE=1 for less-verbose output'
+        Interaction.log(msg1)
+        Interaction.log(msg2)
+    elif trace:
+        msg1 = 'info: debug level 1'
+        msg2 = 'info: set GIT_COLA_TRACE=2 for trace mode'
+        Interaction.log(msg1)
+        Interaction.log(msg2)
 
 
 class ApplicationContext(object):
