@@ -8,12 +8,13 @@ import struct
 from binascii import unhexlify
 from os.path import join
 
-from cola import core
-from cola import git
-from cola import observable
-from cola.decorators import memoize
-from cola.git import STDOUT
-from cola.compat import ustr
+from . import core
+from . import git
+from . import observable
+from .compat import int_types
+from .decorators import memoize
+from .git import STDOUT
+from .compat import ustr
 
 BUILTIN_READER = os.environ.get('GIT_COLA_BUILTIN_CONFIG_READER', False)
 
@@ -22,6 +23,7 @@ _USER_XDG_CONFIG = core.expanduser(
         join(core.getenv('XDG_CONFIG_HOME', join('~', '.config')),
              'git', 'config'))
 
+
 @memoize
 def current():
     """Return the GitConfig singleton."""
@@ -30,10 +32,13 @@ def current():
 
 def _stat_info():
     # Try /etc/gitconfig as a fallback for the system config
-    paths = (('system', '/etc/gitconfig'),
+    paths = [('system', '/etc/gitconfig'),
              ('user', _USER_XDG_CONFIG),
-             ('user', _USER_CONFIG),
-             ('repo', git.current().git_path('config')))
+             ('user', _USER_CONFIG)]
+    config = git.current().git_path('config')
+    if config:
+        paths.append(('repo', config))
+
     statinfo = []
     for category, path in paths:
         try:
@@ -45,10 +50,13 @@ def _stat_info():
 
 def _cache_key():
     # Try /etc/gitconfig as a fallback for the system config
-    paths = ('/etc/gitconfig',
+    paths = ['/etc/gitconfig',
              _USER_XDG_CONFIG,
-             _USER_CONFIG,
-             git.current().git_path('config'))
+             _USER_CONFIG]
+    config = git.current().git_path('config')
+    if config:
+        paths.append(config)
+
     mtimes = []
     for path in paths:
         try:
@@ -142,7 +150,7 @@ class GitConfig(observable.Observable):
         """
         # Try the git config in git's installation prefix
         statinfo = _stat_info()
-        self._configs = map(lambda x: x[1], statinfo)
+        self._configs = [x[1] for x in statinfo]
         self._config_files = {}
         for (cat, path, mtime) in statinfo:
             self._config_files[cat] = path
@@ -219,8 +227,10 @@ class GitConfig(observable.Observable):
         header_subkey = re.compile(r'^\[(\s+) "(\s+)"\]$')
 
         with core.xopen(path, 'rt') as f:
-            lines = filter(bool, [line.strip() for line in f.readlines()])
+            file_lines = f.readlines()
 
+        stripped_lines = [line.strip() for line in file_lines]
+        lines = [line for line in stripped_lines if bool(line)]
         prefix = ''
         for line in lines:
             if line.startswith('#'):
@@ -267,6 +277,15 @@ class GitConfig(observable.Observable):
         """Return the string value for a config key."""
         return self._get(self._all, key, default)
 
+    def get_all(self, key):
+        """Return all values for a key"""
+        status, out, err = self.git.config(key, z=True, get_all=True)
+        if status == 0:
+            result = [x for x in out.rstrip(chr(0)).split(chr(0)) if x]
+        else:
+            result = []
+        return result
+
     def get_user(self, key, default=None):
         return self._get(self._user, key, default)
 
@@ -282,22 +301,31 @@ class GitConfig(observable.Observable):
                 return 'true'
             else:
                 return 'false'
-        if type(value) is int:
+        if isinstance(value, int_types):
             return ustr(value)
         return value
 
     def set_user(self, key, value):
-        msg = self.message_user_config_changed
+        if value is None:
+            self.unset_user(key)
+            return
         self.git.config('--global', key, self.python_to_git(value))
         self.update()
+        msg = self.message_user_config_changed
         self.notify_observers(msg, key, value)
 
     def set_repo(self, key, value):
-        msg = self.message_repo_config_changed
         self.git.config(key, self.python_to_git(value))
         self.update()
+        msg = self.message_repo_config_changed
         self.notify_observers(msg, key, value)
 
+    def unset_user(self, key):
+        self.git.config('--global', '--unset', key)
+        self.update()
+        msg = self.message_repo_config_changed
+        self.notify_observers(msg, key, None)
+
     def find(self, pat):
         pat = pat.lower()
         match = fnmatch.fnmatch
@@ -357,26 +385,27 @@ class GitConfig(observable.Observable):
         prefix = len('guitool.%s.' % name)
         guitools = self.find('guitool.%s.*' % name)
         return dict([(key[prefix:], value)
-                        for (key, value) in guitools.items()])
+                     for (key, value) in guitools.items()])
 
     def get_guitool_names(self):
         guitools = self.find('guitool.*.cmd')
         prefix = len('guitool.')
         suffix = len('.cmd')
         return sorted([name[prefix:-suffix]
-                        for (name, cmd) in guitools.items()])
+                       for (name, cmd) in guitools.items()])
 
     def get_guitool_names_and_shortcuts(self):
         """Return guitool names and their configured shortcut"""
         names = self.get_guitool_names()
-        return [(name, self.get('guitool.%s.shortcut' % name)) for name in names]
+        return [(name, self.get('guitool.%s.shortcut' % name))
+                for name in names]
 
     def terminal(self):
         term = self.get('cola.terminal', None)
         if not term:
             # find a suitable default terminal
-            term = 'xterm -e' # for mac osx
-            candidates = ('xfce4-terminal', 'konsole')
+            term = 'xterm -e'  # for mac osx
+            candidates = ('xfce4-terminal', 'konsole', 'gnome-terminal')
             for basename in candidates:
                 if core.exists('/usr/bin/%s' % basename):
                     term = '%s -e' % basename
