@@ -4,16 +4,16 @@ from __future__ import division, absolute_import, unicode_literals
 import re
 from io import StringIO
 
-from cola import core
-from cola import gitcfg
-from cola import utils
-from cola import version
-from cola.git import git
-from cola.git import STDOUT
-from cola.i18n import N_
+from . import core
+from . import gitcfg
+from . import utils
+from . import version
+from .git import git
+from .git import STDOUT
+from .i18n import N_
 
 
-EMPTY_TREE_SHA1 = '4b825dc642cb6eb9a060e54bf8d69288fbee4904'
+EMPTY_TREE_OID = '4b825dc642cb6eb9a060e54bf8d69288fbee4904'
 
 
 class InvalidRepositoryError(Exception):
@@ -36,10 +36,43 @@ def diff_index_filenames(ref):
 def diff_filenames(*args):
     """Return a list of filenames that have been modified"""
     out = git.diff_tree(name_only=True, no_commit_id=True, r=True, z=True,
-                        *args)[STDOUT]
+                        _readonly=True, *args)[STDOUT]
     return _parse_diff_filenames(out)
 
 
+def listdir(dirname, ref='HEAD'):
+    """Get the contents of a directory according to Git
+
+    Query Git for the content of a directory, taking ignored
+    files into account.
+
+    """
+    dirs = []
+    files = []
+
+    # first, parse git ls-tree to get the tracked files
+    # in a list of (type, path) tuples
+    entries = ls_tree(dirname, ref=ref)
+    for entry in entries:
+        if entry[0][0] == 't':  # tree
+            dirs.append(entry[1])
+        else:
+            files.append(entry[1])
+
+    # gather untracked files
+    untracked = untracked_files(paths=[dirname], directory=True)
+    for path in untracked:
+        if path.endswith('/'):
+            dirs.append(path[:-1])
+        else:
+            files.append(path)
+
+    dirs.sort()
+    files.sort()
+
+    return (dirs, files)
+
+
 def diff(args):
     """Return a list of filenames for the given diff arguments
 
@@ -76,7 +109,7 @@ def all_files(*args):
     return sorted([f for f in ls_files.split('\0') if f])
 
 
-class _current_branch:
+class _current_branch(object):
     """Cache for current_branch()"""
     key = None
     value = None
@@ -114,21 +147,21 @@ def current_branch():
 
 def _read_git_head(head, default='master', git=git):
     """Pure-python .git/HEAD reader"""
-    # Legacy .git/HEAD symlinks
-    if core.islink(head):
-        refs_heads = core.realpath(git.git_path('refs', 'heads'))
-        path = core.abspath(head).replace('\\', '/')
-        if path.startswith(refs_heads + '/'):
-            return path[len(refs_heads)+1:]
-
-    # Common .git/HEAD "ref: refs/heads/master" file
-    elif core.isfile(head):
+    # Common .git/HEAD "ref: refs/heads/master" files
+    islink = core.islink(head)
+    if core.isfile(head) and not islink:
         data = core.read(head).rstrip()
         ref_prefix = 'ref: '
         if data.startswith(ref_prefix):
             return data[len(ref_prefix):]
         # Detached head
         return data
+    # Legacy .git/HEAD symlinks
+    elif islink:
+        refs_heads = core.realpath(git.git_path('refs', 'heads'))
+        path = core.abspath(head).replace('\\', '/')
+        if path.startswith(refs_heads + '/'):
+            return path[len(refs_heads)+1:]
 
     return default
 
@@ -146,29 +179,46 @@ def branch_list(remote=False):
         return for_each_ref_basename('refs/heads')
 
 
+def _version_sort():
+    if version.check_git('version-sort'):
+        sort = 'version:refname'
+    else:
+        sort = False
+    return sort
+
+
 def for_each_ref_basename(refs, git=git):
     """Return refs starting with 'refs'."""
-    out = git.for_each_ref(refs, format='%(refname)')[STDOUT]
+    sort = _version_sort()
+    status, out, err = git.for_each_ref(refs, format='%(refname)',
+                                        sort=sort, _readonly=True)
     output = out.splitlines()
-    non_heads = filter(lambda x: not x.endswith('/HEAD'), output)
+    non_heads = [x for x in output if not x.endswith('/HEAD')]
     return list(map(lambda x: x[len(refs) + 1:], non_heads))
 
 
+def _triple(x, y):
+    return (x, len(x) + 1, y)
+
+
 def all_refs(split=False, git=git):
     """Return a tuple of (local branches, remote branches, tags)."""
     local_branches = []
     remote_branches = []
     tags = []
-    triple = lambda x, y: (x, len(x) + 1, y)
+    triple = _triple
     query = (triple('refs/tags', tags),
              triple('refs/heads', local_branches),
              triple('refs/remotes', remote_branches))
-    out = git.for_each_ref(format='%(refname)')[STDOUT]
+    sort = _version_sort()
+    status, out, err = git.for_each_ref(format='%(refname)',
+                                        sort=sort, _readonly=True)
     for ref in out.splitlines():
         for prefix, prefix_len, dst in query:
             if ref.startswith(prefix) and not ref.endswith('/HEAD'):
                 dst.append(ref[prefix_len:])
                 continue
+    tags.reverse()
     if split:
         return local_branches, remote_branches, tags
     else:
@@ -195,14 +245,14 @@ def tracked_branch(branch=None, config=None):
     return None
 
 
-def untracked_files(git=git, paths=None):
+def untracked_files(git=git, paths=None, **kwargs):
     """Returns a sorted list of untracked files."""
 
     if paths is None:
         paths = []
     args = ['--'] + paths
     out = git.ls_files(z=True, others=True, exclude_standard=True,
-                       *args)[STDOUT]
+                       *args, **kwargs)[STDOUT]
     if out:
         return out[:-1].split('\0')
     return []
@@ -210,19 +260,23 @@ def untracked_files(git=git, paths=None):
 
 def tag_list():
     """Return a list of tags."""
-    return list(reversed(for_each_ref_basename('refs/tags')))
+    result = for_each_ref_basename('refs/tags')
+    result.reverse()
+    return result
 
 
 def log(git, *args, **kwargs):
     return git.log(no_color=True, no_abbrev_commit=True,
-                   no_ext_diff=True, *args, **kwargs)[STDOUT]
+                   no_ext_diff=True, _readonly=True, *args, **kwargs)[STDOUT]
 
 
-def commit_diff(sha1, git=git):
-    return log(git, '-1', sha1, '--') + '\n\n' + sha1_diff(git, sha1)
+def commit_diff(oid, git=git):
+    return log(git, '-1', oid, '--') + '\n\n' + oid_diff(git, oid)
 
 
 _diff_overrides = {}
+
+
 def update_diff_overrides(space_at_eol, space_change,
                           all_space, function_context):
     _diff_overrides['ignore_space_at_eol'] = space_at_eol
@@ -252,30 +306,31 @@ def _add_filename(args, filename):
         args.extend(['--', filename])
 
 
-def sha1_diff(git, sha1, filename=None):
-    """Return the diff for a sha1"""
-    # Naively "$sha1^!" is what we'd like to use but that doesn't
+def oid_diff(git, oid, filename=None):
+    """Return the diff for an oid"""
+    # Naively "$oid^!" is what we'd like to use but that doesn't
     # give the correct result for merges--the diff is reversed.
-    # Be explicit and compare sha1 against its first parent.
-    args = [sha1 + '~', sha1]
+    # Be explicit and compare oid against its first parent.
+    args = [oid + '~', oid]
     opts = common_diff_opts()
     _add_filename(args, filename)
     status, out, err = git.diff(*args, **opts)
     if status != 0:
-        # We probably don't have "$sha1~" because this is the root commit.
+        # We probably don't have "$oid~" because this is the root commit.
         # "git show" is clever enough to handle the root commit.
-        args = [sha1 + '^!']
+        args = [oid + '^!']
         _add_filename(args, filename)
-        status, out, err = git.show(pretty='format:', *args, **opts)
+        status, out, err = git.show(pretty='format:', _readonly=True,
+                                    *args, **opts)
         out = out.lstrip()
     return out
 
 
-def diff_info(sha1, git=git, filename=None):
-    decoded = log(git, '-1', sha1, '--', pretty='format:%b').strip()
+def diff_info(oid, git=git, filename=None):
+    decoded = log(git, '-1', oid, '--', pretty='format:%b').strip()
     if decoded:
         decoded += '\n\n'
-    return decoded + sha1_diff(git, sha1, filename=filename)
+    return decoded + oid_diff(git, oid, filename=filename)
 
 
 def diff_helper(commit=None,
@@ -341,8 +396,7 @@ def extract_diff_header(status, deleted,
     del_tag = 'deleted file mode '
     output = StringIO()
 
-    diff = diffoutput.split('\n')
-    for line in diff:
+    for line in diffoutput.splitlines():
         if not start and '@@' == line[:2] and '@@' in line[2:]:
             start = True
         if start or (deleted and del_tag in line):
@@ -353,7 +407,7 @@ def extract_diff_header(status, deleted,
             elif not suppress_header:
                 output.write(line + '\n')
 
-    result = output.getvalue()
+    result = output.getvalue().rstrip('\n')
     output.close()
 
     if with_diff_header:
@@ -384,14 +438,18 @@ def format_patchsets(to_export, revs, output='patches'):
     # Group the patches into continuous sets
     for idx, rev in enumerate(to_export[1:]):
         # Limit the search to the current neighborhood for efficiency
-        master_idx = revs[cur_master_idx:].index(rev)
-        master_idx += cur_master_idx
+        try:
+            master_idx = revs[cur_master_idx:].index(rev)
+            master_idx += cur_master_idx
+        except ValueError:
+            master_idx = revs.index(rev)
+
         if master_idx == cur_master_idx + 1:
-            patches_to_export[ patchset_idx ].append(rev)
+            patches_to_export[patchset_idx].append(rev)
             cur_master_idx += 1
             continue
         else:
-            patches_to_export.append([ rev ])
+            patches_to_export.append([rev])
             cur_master_idx = master_idx
             patchset_idx += 1
 
@@ -495,7 +553,7 @@ def diff_index(head, cached=True, paths=None):
     status, out, err = git.diff_index(cached=cached, z=True, *args)
     if status != 0:
         # handle git init
-        args[0] = EMPTY_TREE_SHA1
+        args[0] = EMPTY_TREE_OID
         status, out, err = git.diff_index(cached=cached, z=True, *args)
 
     for path, status, is_submodule in _parse_raw_diff(out):
@@ -553,7 +611,7 @@ def _branch_status(branch):
 
 def merge_base(head, ref):
     """Given `ref`, return $(git merge-base ref HEAD)..ref."""
-    return git.merge_base(head, ref)[STDOUT]
+    return git.merge_base(head, ref, _readonly=True)[STDOUT]
 
 
 def merge_base_parent(branch):
@@ -564,26 +622,45 @@ def merge_base_parent(branch):
 
 
 def parse_ls_tree(rev):
-    """Return a list of(mode, type, sha1, path) tuples."""
+    """Return a list of (mode, type, oid, path) tuples."""
     output = []
-    lines = git.ls_tree(rev, r=True)[STDOUT].splitlines()
+    lines = git.ls_tree(rev, r=True, _readonly=True)[STDOUT].splitlines()
     regex = re.compile(r'^(\d+)\W(\w+)\W(\w+)[ \t]+(.*)$')
     for line in lines:
         match = regex.match(line)
         if match:
             mode = match.group(1)
             objtype = match.group(2)
-            sha1 = match.group(3)
+            oid = match.group(3)
             filename = match.group(4)
-            output.append((mode, objtype, sha1, filename,) )
+            output.append((mode, objtype, oid, filename,))
     return output
 
 
+def ls_tree(path, ref='HEAD'):
+    """Return a parsed git ls-tree result for a single directory"""
+
+    result = []
+    status, out, err = git.ls_tree(ref, '--', path, z=True, full_tree=True)
+    if status == 0 and out:
+        for line in out[:-1].split('\0'):
+            # .....6 ...4 ......................................40
+            # 040000 tree c127cde9a0c644a3a8fef449a244f47d5272dfa6	relative
+            # 100644 blob 139e42bf4acaa4927ec9be1ec55a252b97d3f1e2	relative/path
+            # 0..... 7... 12......................................	53
+            # path offset = 6 + 1 + 4 + 1 + 40 + 1 = 53
+            objtype = line[7:11]
+            relpath = line[53:]
+            result.append((objtype, relpath))
+
+    return result
+
 # A regex for matching the output of git(log|rev-list) --pretty=oneline
 REV_LIST_REGEX = re.compile(r'^([0-9a-f]{40}) (.*)$')
 
+
 def parse_rev_list(raw_revs):
-    """Parse `git log --pretty=online` output into (SHA-1, summary) pairs."""
+    """Parse `git log --pretty=online` output into (oid, summary) pairs."""
     revs = []
     for line in raw_revs.splitlines():
         match = REV_LIST_REGEX.match(line)
@@ -595,7 +672,7 @@ def parse_rev_list(raw_revs):
 
 
 def log_helper(all=False, extra_args=None):
-    """Return parallel arrays containing the SHA-1s and summaries."""
+    """Return parallel arrays containing oids and summaries."""
     revs = []
     summaries = []
     args = []
@@ -611,7 +688,7 @@ def log_helper(all=False, extra_args=None):
 
 
 def rev_list_range(start, end):
-    """Return a (SHA-1, summary) pairs between start and end."""
+    """Return (oid, summary) pairs between start and end."""
     revrange = '%s..%s' % (start, end)
     out = git.rev_list(revrange, pretty='oneline')[STDOUT]
     return parse_rev_list(out)
@@ -619,7 +696,7 @@ def rev_list_range(start, end):
 
 def commit_message_path():
     """Return the path to .git/GIT_COLA_MSG"""
-    path = git.git_path("GIT_COLA_MSG")
+    path = git.git_path('GIT_COLA_MSG')
     if core.exists(path):
         return path
     return None
@@ -634,6 +711,13 @@ def merge_message_path():
     return None
 
 
+def prepare_commit_message_hook(config=None):
+    default_hook = git.git_path('hooks', 'cola-prepare-commit-msg')
+    if config is None:
+        config = gitcfg.current()
+    return config.get('cola.preparecommitmessagehook', default_hook)
+
+
 def abort_merge():
     """Abort a merge by reading the tree at HEAD."""
     # Reset the worktree
