@@ -1,42 +1,57 @@
-# Copyright (c) 2008 David Aguilar
-"""This module provides miscellaneous Qt utility functions.
-"""
+# Copyright (c) 2008-2016 David Aguilar
+"""Miscellaneous Qt utility functions."""
 from __future__ import division, absolute_import, unicode_literals
 
-import subprocess
+from qtpy import compat
+from qtpy import QtGui
+from qtpy import QtCore
+from qtpy import QtWidgets
+from qtpy.QtCore import Qt
+from qtpy.QtCore import Signal
 
-from PyQt4 import QtGui
-from PyQt4 import QtCore
-from PyQt4.QtCore import Qt
-from PyQt4.QtCore import SIGNAL
+from . import core
+from . import gitcfg
+from . import hotkeys
+from . import icons
+from . import utils
+from .i18n import N_
+from .interaction import Interaction
+from .compat import int_types
+from .compat import ustr
+from .models import prefs
+from .widgets import defs
 
-from cola import core
-from cola import gitcfg
-from cola import hotkeys
-from cola import icons
-from cola import utils
-from cola import resources
-from cola.i18n import N_
-from cola.interaction import Interaction
-from cola.compat import ustr
-from cola.compat import PY3
-from cola.models import prefs
-from cola.widgets import defs
+
+STRETCH = object()
+SKIPPED = object()
+
+
+def disconnect(signal):
+    """Disconnect signal from all slots"""
+    try:
+        signal.disconnect()
+    except TypeError:  # allow unconnected slots
+        pass
 
 
 def connect_action(action, fn):
-    """Connectc an action to a function"""
-    action.connect(action, SIGNAL('triggered()'), fn)
+    """Connect an action to a function"""
+    action.triggered[bool].connect(lambda x: fn())
 
 
 def connect_action_bool(action, fn):
     """Connect a triggered(bool) action to a function"""
-    action.connect(action, SIGNAL('triggered(bool)'), fn)
+    action.triggered[bool].connect(fn)
 
 
 def connect_button(button, fn):
     """Connect a button to a function"""
-    button.connect(button, SIGNAL('clicked()'), fn)
+    button.pressed.connect(fn)
+
+
+def connect_released(button, fn):
+    """Connect a button to a function"""
+    button.released.connect(fn)
 
 
 def button_action(button, action):
@@ -45,49 +60,51 @@ def button_action(button, action):
 
 
 def connect_toggle(toggle, fn):
-    toggle.connect(toggle, SIGNAL('toggled(bool)'), fn)
+    """Connect a toggle button to a function"""
+    toggle.toggled.connect(fn)
 
 
 def active_window():
-    return QtGui.QApplication.activeWindow()
+    """Return the active window for the current application"""
+    return QtWidgets.QApplication.activeWindow()
 
 
 def hbox(margin, spacing, *items):
-    return box(QtGui.QHBoxLayout, margin, spacing, *items)
+    """Create an HBoxLayout with the specified sizes and items"""
+    return box(QtWidgets.QHBoxLayout, margin, spacing, *items)
 
 
 def vbox(margin, spacing, *items):
-    return box(QtGui.QVBoxLayout, margin, spacing, *items)
+    """Create a VBoxLayout with the specified sizes and items"""
+    return box(QtWidgets.QVBoxLayout, margin, spacing, *items)
 
 
 def buttongroup(*items):
-    group = QtGui.QButtonGroup()
+    """Create a QButtonGroup for the specified items"""
+    group = QtWidgets.QButtonGroup()
     for i in items:
         group.addButton(i)
     return group
 
 
-STRETCH = object()
-SKIPPED = object()
+def set_margin(layout, margin):
+    """Set the content margins for a layout"""
+    layout.setContentsMargins(margin, margin, margin, margin)
 
 
 def box(cls, margin, spacing, *items):
+    """Create a QBoxLayout with the specified sizes and items"""
     stretch = STRETCH
     skipped = SKIPPED
     layout = cls()
-    layout.setMargin(margin)
     layout.setSpacing(spacing)
-
-    if PY3:
-        int_types = (int,)
-    else:
-        int_types = (int, long)
+    set_margin(layout, margin)
 
     for i in items:
-        if isinstance(i, QtGui.QWidget):
+        if isinstance(i, QtWidgets.QWidget):
             layout.addWidget(i)
-        elif isinstance(i, (QtGui.QHBoxLayout, QtGui.QVBoxLayout,
-                            QtGui.QFormLayout, QtGui.QLayout)):
+        elif isinstance(i, (QtWidgets.QHBoxLayout, QtWidgets.QVBoxLayout,
+                            QtWidgets.QFormLayout, QtWidgets.QLayout)):
             layout.addLayout(i)
         elif i is stretch:
             layout.addStretch()
@@ -100,72 +117,160 @@ def box(cls, margin, spacing, *items):
 
 
 def form(margin, spacing, *widgets):
-    layout = QtGui.QFormLayout()
-    layout.setMargin(margin)
+    """Create a QFormLayout with the specified sizes and items"""
+    layout = QtWidgets.QFormLayout()
     layout.setSpacing(spacing)
-    layout.setFieldGrowthPolicy(QtGui.QFormLayout.ExpandingFieldsGrow)
+    layout.setFieldGrowthPolicy(QtWidgets.QFormLayout.ExpandingFieldsGrow)
+    set_margin(layout, margin)
 
-    for idx, (label, widget) in enumerate(widgets):
-        if isinstance(label, (str, ustr)):
-            layout.addRow(label, widget)
+    for idx, (name, widget) in enumerate(widgets):
+        if isinstance(name, (str, ustr)):
+            layout.addRow(name, widget)
         else:
-            layout.setWidget(idx, QtGui.QFormLayout.LabelRole, label)
-            layout.setWidget(idx, QtGui.QFormLayout.FieldRole, widget)
+            layout.setWidget(idx, QtWidgets.QFormLayout.LabelRole, name)
+            layout.setWidget(idx, QtWidgets.QFormLayout.FieldRole, widget)
 
     return layout
 
 
 def grid(margin, spacing, *widgets):
-    layout = QtGui.QGridLayout()
-    layout.setMargin(defs.no_margin)
-    layout.setSpacing(defs.spacing)
+    """Create a QGridLayout with the specified sizes and items"""
+    layout = QtWidgets.QGridLayout()
+    layout.setSpacing(spacing)
+    set_margin(layout, margin)
 
     for row in widgets:
         item = row[0]
-        if isinstance(item, QtGui.QWidget):
+        if isinstance(item, QtWidgets.QWidget):
             layout.addWidget(*row)
-        elif isinstance(item, QtGui.QLayoutItem):
+        elif isinstance(item, QtWidgets.QLayoutItem):
             layout.addItem(*row)
 
     return layout
 
 
 def splitter(orientation, *widgets):
-    layout = QtGui.QSplitter()
+    """Create a spliter over the specified widgets
+
+    :param orientation: Qt.Horizontal or Qt.Vertical
+
+    """
+    layout = QtWidgets.QSplitter()
     layout.setOrientation(orientation)
     layout.setHandleWidth(defs.handle_width)
     layout.setChildrenCollapsible(True)
+
     for idx, widget in enumerate(widgets):
         layout.addWidget(widget)
         layout.setStretchFactor(idx, 1)
 
+    # Workaround for Qt not setting the WA_Hover property for QSplitter
+    # Cf. https://bugreports.qt.io/browse/QTBUG-13768
+    layout.handle(1).setAttribute(Qt.WA_Hover)
+
     return layout
 
 
+def label(text=None, align=None, fmt=None, selectable=True):
+    """Create a QLabel with the specified properties"""
+    widget = QtWidgets.QLabel()
+    if align is not None:
+        widget.setAlignment(align)
+    if fmt is not None:
+        widget.setTextFormat(fmt)
+    if selectable:
+        widget.setTextInteractionFlags(Qt.TextBrowserInteraction)
+        widget.setOpenExternalLinks(True)
+    if text:
+        widget.setText(text)
+    return widget
+
+
+def textbrowser(text=None):
+    """Create a QTextBrowser for the specified text"""
+    widget = QtWidgets.QTextBrowser()
+    widget.setOpenExternalLinks(True)
+    if text:
+        widget.setText(text)
+    return widget
+
+
+def add_completer(widget, items):
+    """Add simple completion to a widget"""
+    completer = QtWidgets.QCompleter(items, widget)
+    completer.setCaseSensitivity(Qt.CaseInsensitive)
+    completer.setCompletionMode(QtWidgets.QCompleter.InlineCompletion)
+    widget.setCompleter(completer)
+
+
 def prompt(msg, title=None, text=''):
     """Presents the user with an input widget and returns the input."""
     if title is None:
         title = msg
-    result = QtGui.QInputDialog.getText(active_window(), msg, title,
-                                        QtGui.QLineEdit.Normal, text)
-    return (ustr(result[0]), result[1])
+    result = QtWidgets.QInputDialog.getText(
+            active_window(), msg, title,
+            QtWidgets.QLineEdit.Normal, text)
+    return (result[0], result[1])
 
 
-def create_listwidget_item(text, filename):
-    """Creates a QListWidgetItem with text and the icon at filename."""
-    item = QtGui.QListWidgetItem()
-    item.setIcon(QtGui.QIcon(filename))
-    item.setIconSize(QtCore.QSize(defs.small_icon, defs.small_icon))
-    item.setText(text)
-    return item
+def prompt_n(msg, inputs):
+    """Presents the user with N input widgets and returns the results"""
+    dialog = QtWidgets.QDialog(active_window())
+    dialog.setWindowModality(Qt.WindowModal)
+    dialog.setWindowTitle(msg)
+
+    long_value = msg
+    for k, v in inputs:
+        if len(k + v) > len(long_value):
+            long_value = k + v
+
+    metrics = QtGui.QFontMetrics(dialog.font())
+    min_width = metrics.width(long_value) + 100
+    if min_width > 720:
+        min_width = 720
+    dialog.setMinimumWidth(min_width)
+
+    ok_b = ok_button(msg, enabled=False)
+    close_b = close_button()
+
+    form_widgets = []
+
+    def get_values():
+        return [pair[1].text().strip() for pair in form_widgets]
+
+    for name, value in inputs:
+        lineedit = QtWidgets.QLineEdit()
+        # Enable the OK button only when all fields have been populated
+        lineedit.textChanged.connect(
+                lambda x: ok_b.setEnabled(all(get_values())))
+        if value:
+            lineedit.setText(value)
+        form_widgets.append((name, lineedit))
+
+    # layouts
+    form_layout = form(defs.no_margin, defs.button_spacing, *form_widgets)
+    button_layout = hbox(defs.no_margin, defs.button_spacing,
+                         STRETCH, close_b, ok_b)
+    main_layout = vbox(defs.margin, defs.button_spacing,
+                       form_layout, button_layout)
+    dialog.setLayout(main_layout)
+
+    # connections
+    connect_button(ok_b, dialog.accept)
+    connect_button(close_b, dialog.reject)
+
+    accepted = dialog.exec_() == QtWidgets.QDialog.Accepted
+    text = get_values()
+    ok = accepted and all(text)
+    return (ok, text)
 
 
-class TreeWidgetItem(QtGui.QTreeWidgetItem):
+class TreeWidgetItem(QtWidgets.QTreeWidgetItem):
 
     TYPE = QtGui.QStandardItem.UserType + 101
 
     def __init__(self, path, icon, deleted):
-        QtGui.QTreeWidgetItem.__init__(self)
+        QtWidgets.QTreeWidgetItem.__init__(self)
         self.path = path
         self.deleted = deleted
         self.setIcon(0, icons.from_name(icon))
@@ -183,12 +288,16 @@ def paths_from_indexes(model, indexes,
     return paths_from_items(items, item_type=item_type, item_filter=item_filter)
 
 
+def _true_filter(x):
+    return True
+
+
 def paths_from_items(items,
                      item_type=TreeWidgetItem.TYPE,
                      item_filter=None):
     """Return a list of paths from a list of items"""
     if item_filter is None:
-        item_filter = lambda x: True
+        item_filter = _true_filter
     return [i.path for i in items
             if i.type() == item_type and item_filter(i)]
 
@@ -197,17 +306,17 @@ def confirm(title, text, informative_text, ok_text,
             icon=None, default=True,
             cancel_text=None, cancel_icon=None):
     """Confirm that an action should take place"""
-    msgbox = QtGui.QMessageBox(active_window())
+    msgbox = QtWidgets.QMessageBox(active_window())
     msgbox.setWindowModality(Qt.WindowModal)
     msgbox.setWindowTitle(title)
     msgbox.setText(text)
     msgbox.setInformativeText(informative_text)
 
     icon = icons.mkicon(icon, icons.ok)
-    ok = msgbox.addButton(ok_text, QtGui.QMessageBox.ActionRole)
+    ok = msgbox.addButton(ok_text, QtWidgets.QMessageBox.ActionRole)
     ok.setIcon(icon)
 
-    cancel = msgbox.addButton(QtGui.QMessageBox.Cancel)
+    cancel = msgbox.addButton(QtWidgets.QMessageBox.Cancel)
     cancel_icon = icons.mkicon(cancel_icon, icons.close)
     cancel.setIcon(cancel_icon)
     if cancel_text:
@@ -221,24 +330,24 @@ def confirm(title, text, informative_text, ok_text,
     return msgbox.clickedButton() == ok
 
 
-class ResizeableMessageBox(QtGui.QMessageBox):
+class ResizeableMessageBox(QtWidgets.QMessageBox):
 
     def __init__(self, parent):
-        QtGui.QMessageBox.__init__(self, parent)
+        QtWidgets.QMessageBox.__init__(self, parent)
         self.setMouseTracking(True)
         self.setSizeGripEnabled(True)
 
     def event(self, event):
-        res = QtGui.QMessageBox.event(self, event)
+        res = QtWidgets.QMessageBox.event(self, event)
         event_type = event.type()
         if (event_type == QtCore.QEvent.MouseMove or
                 event_type == QtCore.QEvent.MouseButtonPress):
             maxi = QtCore.QSize(defs.max_size, defs.max_size)
             self.setMaximumSize(maxi)
-            text = self.findChild(QtGui.QTextEdit)
+            text = self.findChild(QtWidgets.QTextEdit)
             if text is not None:
-                expand = QtGui.QSizePolicy.Expanding
-                text.setSizePolicy(QtGui.QSizePolicy(expand, expand))
+                expand = QtWidgets.QSizePolicy.Expanding
+                text.setSizePolicy(QtWidgets.QSizePolicy(expand, expand))
                 text.setMaximumSize(maxi)
         return res
 
@@ -251,9 +360,9 @@ def critical(title, message=None, details=None):
     mbox.setWindowTitle(title)
     mbox.setTextFormat(Qt.PlainText)
     mbox.setText(message)
-    mbox.setIcon(QtGui.QMessageBox.Critical)
-    mbox.setStandardButtons(QtGui.QMessageBox.Close)
-    mbox.setDefaultButton(QtGui.QMessageBox.Close)
+    mbox.setIcon(QtWidgets.QMessageBox.Critical)
+    mbox.setStandardButtons(QtWidgets.QMessageBox.Close)
+    mbox.setDefaultButton(QtWidgets.QMessageBox.Close)
     if details:
         mbox.setDetailedText(details)
     mbox.exec_()
@@ -263,9 +372,9 @@ def information(title, message=None, details=None, informative_text=None):
     """Show information with the provided title and message."""
     if message is None:
         message = title
-    mbox = QtGui.QMessageBox(active_window())
-    mbox.setStandardButtons(QtGui.QMessageBox.Close)
-    mbox.setDefaultButton(QtGui.QMessageBox.Close)
+    mbox = QtWidgets.QMessageBox(active_window())
+    mbox.setStandardButtons(QtWidgets.QMessageBox.Close)
+    mbox.setDefaultButton(QtWidgets.QMessageBox.Close)
     mbox.setWindowTitle(title)
     mbox.setWindowModality(Qt.WindowModal)
     mbox.setTextFormat(Qt.PlainText)
@@ -283,16 +392,18 @@ def information(title, message=None, details=None, informative_text=None):
 def question(title, msg, default=True):
     """Launches a QMessageBox question with the provided title and message.
     Passing "default=False" will make "No" the default choice."""
-    yes = QtGui.QMessageBox.Yes
-    no = QtGui.QMessageBox.No
+    yes = QtWidgets.QMessageBox.Yes
+    no = QtWidgets.QMessageBox.No
     buttons = yes | no
     if default:
         default = yes
     else:
         default = no
-    result = (QtGui.QMessageBox
-                   .question(active_window(), title, msg, buttons, default))
-    return result == QtGui.QMessageBox.Yes
+
+    parent = active_window()
+    MessageBox = QtWidgets.QMessageBox
+    result = MessageBox.question(parent, title, msg, buttons, default)
+    return result == QtWidgets.QMessageBox.Yes
 
 
 def tree_selection(tree_item, items):
@@ -346,30 +457,38 @@ def selected_items(list_widget, items):
 
 def open_file(title, directory=None):
     """Creates an Open File dialog and returns a filename."""
-    return ustr(QtGui.QFileDialog
-                        .getOpenFileName(active_window(), title, directory))
+    result = compat.getopenfilename(parent=active_window(),
+                                    caption=title,
+                                    basedir=directory)
+    return result[0]
 
 
-def open_files(title, directory=None, filter=None):
+def open_files(title, directory=None, filters=''):
     """Creates an Open File dialog and returns a list of filenames."""
-    return (QtGui.QFileDialog
-            .getOpenFileNames(active_window(), title, directory, filter))
+    result = compat.getopenfilenames(parent=active_window(),
+                                     caption=title,
+                                     basedir=directory,
+                                     filters=filters)
+    return result[0]
 
 
-def opendir_dialog(title, path):
+def opendir_dialog(caption, path):
     """Prompts for a directory path"""
 
-    flags = (QtGui.QFileDialog.ShowDirsOnly |
-             QtGui.QFileDialog.DontResolveSymlinks)
-    return ustr(QtGui.QFileDialog
-                     .getExistingDirectory(active_window(),
-                                           title, path, flags))
+    options = (QtWidgets.QFileDialog.ShowDirsOnly |
+               QtWidgets.QFileDialog.DontResolveSymlinks)
+    return compat.getexistingdirectory(parent=active_window(),
+                                       caption=caption,
+                                       basedir=path,
+                                       options=options)
 
 
 def save_as(filename, title='Save As...'):
     """Creates a Save File dialog and returns a filename."""
-    return ustr(QtGui.QFileDialog
-                        .getSaveFileName(active_window(), title, filename))
+    result = compat.getsavefilename(parent=active_window(),
+                                    caption=title,
+                                    basedir=filename)
+    return result[0]
 
 
 def copy_path(filename, absolute=True):
@@ -385,7 +504,7 @@ def set_clipboard(text):
     """Sets the copy/paste buffer to text."""
     if not text:
         return
-    clipboard = QtGui.QApplication.clipboard()
+    clipboard = QtWidgets.QApplication.clipboard()
     clipboard.setText(text, QtGui.QClipboard.Clipboard)
     clipboard.setText(text, QtGui.QClipboard.Selection)
     persist_clipboard()
@@ -402,9 +521,9 @@ def persist_clipboard():
     C.f. https://stackoverflow.com/questions/2007103/how-can-i-disable-clear-of-clipboard-on-exit-of-pyqt4-application
 
     """
-    clipboard = QtGui.QApplication.clipboard()
+    clipboard = QtWidgets.QApplication.clipboard()
     event = QtCore.QEvent(QtCore.QEvent.Clipboard)
-    QtGui.QApplication.sendEvent(clipboard, event)
+    QtWidgets.QApplication.sendEvent(clipboard, event)
 
 
 def add_action_bool(widget, text, fn, checked, *shortcuts):
@@ -425,7 +544,9 @@ def add_action_with_status_tip(widget, text, tip, fn, *shortcuts):
 
 
 def _add_action(widget, text, tip, fn, connect, *shortcuts):
-    action = QtGui.QAction(text, widget)
+    action = QtWidgets.QAction(text, widget)
+    if hasattr(action, 'setIconVisibleInMenu'):
+        action.setIconVisibleInMenu(True)
     if tip:
         action.setStatusTip(tip)
     connect(action, fn)
@@ -439,10 +560,10 @@ def _add_action(widget, text, tip, fn, connect, *shortcuts):
 
 def set_selected_item(widget, idx):
     """Sets a the currently selected item to the item at index idx."""
-    if type(widget) is QtGui.QTreeWidget:
+    if type(widget) is QtWidgets.QTreeWidget:
         item = widget.topLevelItem(idx)
         if item:
-            widget.setItemSelected(item, True)
+            item.setSelected(True)
             widget.setCurrentItem(item)
 
 
@@ -460,7 +581,6 @@ def set_items(widget, items):
     add_items(widget, items)
 
 
-
 def create_treeitem(filename, staged=False, deleted=False, untracked=False):
     """Given a filename, return a TreeWidgetItem for a status widget
 
@@ -468,7 +588,8 @@ def create_treeitem(filename, staged=False, deleted=False, untracked=False):
 
     """
     icon_name = icons.status(filename, deleted, staged, untracked)
-    return TreeWidgetItem(filename, resources.icon(icon_name), deleted=deleted)
+    return TreeWidgetItem(filename, icons.name_from_basename(icon_name),
+                          deleted=deleted)
 
 
 def add_close_action(widget):
@@ -477,20 +598,31 @@ def add_close_action(widget):
                       widget.close, hotkeys.CLOSE, hotkeys.QUIT)
 
 
+def app():
+    """Return the current application"""
+    return QtWidgets.QApplication.instance()
+
+
+def desktop():
+    """Return the desktop"""
+    return app().desktop()
+
+
 def center_on_screen(widget):
     """Move widget to the center of the default screen"""
-    desktop = QtGui.QApplication.instance().desktop()
-    rect = desktop.screenGeometry(QtGui.QCursor().pos())
+    desk = desktop()
+    rect = desk.screenGeometry(QtGui.QCursor().pos())
     cy = rect.height()//2
     cx = rect.width()//2
     widget.move(cx - widget.width()//2, cy - widget.height()//2)
 
 
-def default_size(parent, width, height):
+def default_size(parent, width, height, use_parent_height=True):
     """Return the parent's size, or the provided defaults"""
     if parent is not None:
         width = parent.width()
-        height = parent.height()
+        if use_parent_height:
+            height = parent.height()
     return (width, height)
 
 
@@ -506,8 +638,7 @@ def default_monospace_font():
 def diff_font_str():
     font_str = gitcfg.current().get(prefs.FONTDIFF)
     if font_str is None:
-        font = default_monospace_font()
-        font_str = ustr(font.toString())
+        font_str = default_monospace_font().toString()
     return font_str
 
 
@@ -524,7 +655,7 @@ def font(string):
 def create_button(text='', layout=None, tooltip=None, icon=None,
                   enabled=True, default=False):
     """Create a button, set its title, and add it to the parent."""
-    button = QtGui.QPushButton()
+    button = QtWidgets.QPushButton()
     button.setCursor(Qt.PointingHandCursor)
     if text:
         button.setText(text)
@@ -543,7 +674,7 @@ def create_button(text='', layout=None, tooltip=None, icon=None,
 
 
 def create_action_button(tooltip=None, icon=None):
-    button = QtGui.QPushButton()
+    button = QtWidgets.QPushButton()
     button.setCursor(Qt.PointingHandCursor)
     button.setFlat(True)
     if tooltip is not None:
@@ -554,9 +685,10 @@ def create_action_button(tooltip=None, icon=None):
     return button
 
 
-def ok_button(text, default=False, enabled=True):
-    return create_button(text=text, icon=icons.ok(),
-                         default=default, enabled=enabled)
+def ok_button(text, default=True, enabled=True, icon=None):
+    if icon is None:
+        icon = icons.ok()
+    return create_button(text=text, icon=icon, default=default, enabled=enabled)
 
 
 def close_button():
@@ -574,8 +706,9 @@ def refresh_button(enabled=True, default=False):
 
 
 def hide_button_menu_indicator(button):
-    cls = type(button)
-    name = cls.__name__
+    """Hide the menu indicator icon on buttons"""
+
+    name = button.__class__.__name__
     stylesheet = """
         %(name)s::menu-indicator {
             image: none;
@@ -587,84 +720,41 @@ def hide_button_menu_indicator(button):
                 border-style: none;
             }
         """
-    button.setStyleSheet(stylesheet % {'name': name})
+    button.setStyleSheet(stylesheet % dict(name=name))
 
 
 def checkbox(text='', tooltip='', checked=None):
-    cb = QtGui.QCheckBox()
-    if text:
-        cb.setText(text)
-    if tooltip:
-        cb.setToolTip(tooltip)
-    if checked is not None:
-        cb.setChecked(checked)
-
-    url = icons.check_name()
-    style = """
-        QCheckBox::indicator {
-            width: %(size)dpx;
-            height: %(size)dpx;
-        }
-        QCheckBox::indicator::unchecked {
-            border: %(border)dpx solid #999;
-            background: #fff;
-        }
-        QCheckBox::indicator::checked {
-            image: url(%(url)s);
-            border: %(border)dpx solid black;
-            background: #fff;
-        }
-    """ % dict(size=defs.checkbox, border=defs.border, url=url)
-    cb.setStyleSheet(style)
-
-    return cb
+    """Create a checkbox"""
+    return _checkbox(QtWidgets.QCheckBox, text, tooltip, checked)
 
 
 def radio(text='', tooltip='', checked=None):
-    rb = QtGui.QRadioButton()
+    """Create a radio button"""
+    return _checkbox(QtWidgets.QRadioButton, text, tooltip, checked)
+
+
+def _checkbox(cls, text, tooltip, checked):
+    """Create a widget and apply properties"""
+    widget = cls()
     if text:
-        rb.setText(text)
+        widget.setText(text)
     if tooltip:
-        rb.setToolTip(tooltip)
+        widget.setToolTip(tooltip)
     if checked is not None:
-        rb.setChecked(checked)
-
-    size = defs.checkbox
-    radius = size / 2
-    border = defs.radio_border
-    url = icons.dot_name()
-    style = """
-        QRadioButton::indicator {
-            width: %(size)dpx;
-            height: %(size)dpx;
-        }
-        QRadioButton::indicator::unchecked {
-            background: #fff;
-            border: %(border)dpx solid #999;
-            border-radius: %(radius)dpx;
-        }
-        QRadioButton::indicator::checked {
-            image: url(%(url)s);
-            background: #fff;
-            border: %(border)dpx solid black;
-            border-radius: %(radius)dpx;
-        }
-    """ % dict(size=size, radius=radius, border=border, url=url)
-    rb.setStyleSheet(style)
-
-    return rb
+        widget.setChecked(checked)
+    return widget
 
 
-class DockTitleBarWidget(QtGui.QWidget):
+class DockTitleBarWidget(QtWidgets.QWidget):
 
     def __init__(self, parent, title, stretch=True):
-        QtGui.QWidget.__init__(self, parent)
-        self.label = label = QtGui.QLabel()
-        font = label.font()
+        QtWidgets.QWidget.__init__(self, parent)
+        self.setAutoFillBackground(True)
+        self.label = qlabel = QtWidgets.QLabel(title, self)
+        font = qlabel.font()
         font.setBold(True)
-        label.setFont(font)
-        label.setText(title)
-        label.setCursor(Qt.OpenHandCursor)
+        qlabel.setFont(font)
+        qlabel.setCursor(Qt.OpenHandCursor)
 
         self.close_button = create_action_button(
             tooltip=N_('Close'), icon=icons.close())
@@ -680,7 +770,7 @@ class DockTitleBarWidget(QtGui.QWidget):
             separator = SKIPPED
 
         self.main_layout = hbox(defs.small_margin, defs.spacing,
-                                label, separator, self.corner_layout,
+                                qlabel, separator, self.corner_layout,
                                 self.toggle_button, self.close_button)
         self.setLayout(self.main_layout)
 
@@ -710,11 +800,12 @@ class DockTitleBarWidget(QtGui.QWidget):
 
 def create_dock(title, parent, stretch=True):
     """Create a dock widget and set it up accordingly."""
-    dock = QtGui.QDockWidget(parent)
+    dock = QtWidgets.QDockWidget(parent)
     dock.setWindowTitle(title)
     dock.setObjectName(title)
     titlebar = DockTitleBarWidget(dock, title, stretch=stretch)
     dock.setTitleBarWidget(titlebar)
+    dock.setAutoFillBackground(True)
     if hasattr(parent, 'dockwidgets'):
         parent.dockwidgets.append(dock)
     return dock
@@ -722,13 +813,12 @@ def create_dock(title, parent, stretch=True):
 
 def create_menu(title, parent):
     """Create a menu and set its title."""
-    qmenu = QtGui.QMenu(parent)
-    qmenu.setTitle(title)
+    qmenu = QtWidgets.QMenu(title, parent)
     return qmenu
 
 
 def create_toolbutton(text=None, layout=None, tooltip=None, icon=None):
-    button = QtGui.QToolButton()
+    button = QtWidgets.QToolButton()
     button.setAutoRaise(True)
     button.setAutoFillBackground(True)
     button.setCursor(Qt.PointingHandCursor)
@@ -761,7 +851,7 @@ def mimedata_from_paths(paths):
     # gnome-terminal expects utf-16 encoded text, but other terminals,
     # e.g. terminator, prefer utf-8, so allow cola.dragencoding
     # to override the default.
-    paths_text = subprocess.list2cmdline(abspaths)
+    paths_text = core.list2cmdline(abspaths)
     encoding = gitcfg.current().get('cola.dragencoding', 'utf-16')
     moz_text = core.encode(paths_text, encoding=encoding)
     mimedata.setData('text/x-moz-url', moz_text)
@@ -790,6 +880,11 @@ class BlockSignals(object):
             w.blockSignals(self.values[w])
 
 
+class Channel(QtCore.QObject):
+    finished = Signal(object)
+    result = Signal(object)
+
+
 class Task(QtCore.QRunnable):
     """Disable auto-deletion to avoid gc issues
 
@@ -798,30 +893,26 @@ class Task(QtCore.QRunnable):
 
     """
 
-    FINISHED = SIGNAL('TASK_FINISHED')
-    RESULT = SIGNAL('TASK_RESULT')
-
-    def __init__(self, parent, *args, **kwargs):
+    def __init__(self, parent):
         QtCore.QRunnable.__init__(self)
 
-        self.channel = QtCore.QObject(parent)
+        self.channel = Channel()
         self.result = None
         self.setAutoDelete(False)
 
     def run(self):
         self.result = self.task()
-        self.channel.emit(self.RESULT, self.result)
+        self.channel.result.emit(self.result)
         self.done()
 
     def task(self):
-        pass
+        return None
 
     def done(self):
-        self.channel.emit(self.FINISHED, self)
+        self.channel.finished.emit(self)
 
     def connect(self, handler):
-        self.channel.connect(self.channel, self.RESULT,
-                             handler, Qt.QueuedConnection)
+        self.channel.result.connect(handler, type=Qt.QueuedConnection)
 
 
 class SimpleTask(Task):
@@ -856,11 +947,10 @@ class RunTask(QtCore.QObject):
         task_id = id(task)
         self.task_details[task_id] = (progress, finish)
 
-        self.connect(task.channel, Task.FINISHED, self.finish,
-                     Qt.QueuedConnection)
+        task.channel.finished.connect(self.finish, type=Qt.QueuedConnection)
         self.threadpool.start(task)
 
-    def finish(self, task, *args, **kwargs):
+    def finish(self, task):
         task_id = id(task)
         try:
             self.tasks.remove(task)
@@ -876,20 +966,35 @@ class RunTask(QtCore.QObject):
             progress.hide()
 
         if finish is not None:
-            finish(task, *args, **kwargs)
+            finish(task)
 
 
 # Syntax highlighting
 
+def rgb(r, g, b):
+    color = QtGui.QColor()
+    color.setRgb(r, g, b)
+    return color
+
+
 def rgba(r, g, b, a=255):
-    c = QtGui.QColor()
-    c.setRgb(r, g, b)
-    c.setAlpha(a)
-    return c
+    color = rgb(r, g, b)
+    color.setAlpha(a)
+    return color
 
 
 def RGB(args):
-    return rgba(*args)
+    return rgb(*args)
+
+
+def rgb_css(color):
+    """Convert a QColor into an rgb(int, int, int) CSS string"""
+    return 'rgb(%d, %d, %d)' % (color.red(), color.green(), color.blue())
+
+
+def rgb_hex(color):
+    """Convert a QColor into a hex aabbcc string"""
+    return '%x%x%x' % (color.red(), color.green(), color.blue())
 
 
 def make_format(fg=None, bg=None, bold=False):
