@@ -1,52 +1,64 @@
 from __future__ import division, absolute_import, unicode_literals
 
-from PyQt4 import QtGui
-from PyQt4 import QtCore
-from PyQt4.QtCore import Qt
-from PyQt4.QtCore import SIGNAL
+from qtpy.QtCore import Qt
+from qtpy.QtCore import Signal
+from qtpy import QtCore
+from qtpy import QtGui
+from qtpy import QtWidgets
 
-from cola import cmds
-from cola import core
-from cola import difftool
-from cola import gitcmds
-from cola import hotkeys
-from cola import icons
-from cola import utils
-from cola import qtutils
-from cola.cmds import BaseCommand
-from cola.git import git
-from cola.i18n import N_
-from cola.interaction import Interaction
-from cola.models import main
-from cola.models.browse import GitRepoEntryStore
-from cola.models.browse import GitRepoModel
-from cola.models.browse import GitRepoNameItem
-from cola.models.selection import State
-from cola.models.selection import selection_model
-from cola.widgets import defs
-from cola.widgets import standard
-from cola.widgets.selectcommits import select_commits
-from cola.compat import ustr
+from ..models.browse import GitRepoModel
+from ..models.browse import GitRepoNameItem
+from ..models.selection import State
+from ..models.selection import selection_model
+from ..cmds import BaseCommand
+from ..git import git
+from ..i18n import N_
+from ..interaction import Interaction
+from ..models import browse
+from ..models import main
+from ..compat import ustr
+from .. import cmds
+from .. import core
+from .. import gitcmds
+from .. import hotkeys
+from .. import icons
+from .. import utils
+from .. import qtutils
+from .selectcommits import select_commits
+from . import common
+from . import defs
+from . import standard
 
 
-def worktree_browser_widget(parent, update=True, settings=None):
-    """Return a widget for immediate use."""
+def worktree_browser(parent=None, update=True, settings=None, show=False):
+    """Create a new worktree browser"""
     view = Browser(parent, update=update, settings=settings)
-    view.tree.setModel(GitRepoModel(view.tree))
-    view.ctl = BrowserController(view.tree)
+    model = GitRepoModel(view.tree)
+    view.set_model(model)
     if update:
-        view.tree.refresh()
+        view.refresh()
+    if show:
+        view.show()
     return view
 
 
-def worktree_browser(update=True, settings=None):
-    """Launch a new worktree browser session."""
-    view = worktree_browser_widget(None, update=update, settings=settings)
-    view.show()
-    return view
+def save_path(path, model):
+    """Choose an output filename based on the selected path"""
+    filename = qtutils.save_as(model.filename)
+    if filename:
+        model.filename = filename
+        cmds.do(SaveBlob, model)
+        result = True
+    else:
+        result = False
+    return result
 
 
 class Browser(standard.Widget):
+    updated = Signal()
+
+    # Read-only mode property
+    mode = property(lambda self: self.model.mode)
 
     def __init__(self, parent, update=True, settings=None):
         standard.Widget.__init__(self, parent)
@@ -55,8 +67,8 @@ class Browser(standard.Widget):
         self.mainlayout = qtutils.hbox(defs.no_margin, defs.spacing, self.tree)
         self.setLayout(self.mainlayout)
 
-        self.connect(self, SIGNAL('updated()'),
-                     self._updated_callback, Qt.QueuedConnection)
+        self.updated.connect(self._updated_callback, type=Qt.QueuedConnection)
+
         self.model = main.model()
         self.model.add_observer(self.model.message_updated, self.model_updated)
         if parent is None:
@@ -64,16 +76,19 @@ class Browser(standard.Widget):
         if update:
             self.model_updated()
 
-        # Restore saved settings
-        if not self.restore_state(settings=settings):
-            self.resize(720, 420)
+        self.init_state(settings, self.resize, 720, 420)
 
-    # Read-only mode property
-    mode = property(lambda self: self.model.mode)
+    def set_model(self, model):
+        """Set the model"""
+        self.tree.set_model(model)
+
+    def refresh(self):
+        """Refresh the model triggering view updates"""
+        self.tree.refresh()
 
     def model_updated(self):
         """Update the title with the current branch and directory name."""
-        self.emit(SIGNAL('updated()'))
+        self.updated.emit()
 
     def _updated_callback(self):
         branch = self.model.currentbranch
@@ -93,6 +108,9 @@ class Browser(standard.Widget):
 class RepoTreeView(standard.TreeView):
     """Provides a filesystem-like view of a git repository."""
 
+    about_to_update = Signal()
+    updated = Signal()
+
     def __init__(self, parent):
         standard.TreeView.__init__(self, parent)
 
@@ -101,10 +119,12 @@ class RepoTreeView(standard.TreeView):
         self.saved_open_folders = set()
         self.restoring_selection = False
 
+        self.info_event_type = browse.GitRepoInfoEvent.TYPE
+
         self.setDragEnabled(True)
         self.setRootIsDecorated(False)
         self.setSortingEnabled(False)
-        self.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
+        self.setSelectionMode(self.ExtendedSelection)
 
         # Observe model updates
         model = main.model()
@@ -112,74 +132,98 @@ class RepoTreeView(standard.TreeView):
                            self.emit_about_to_update)
         model.add_observer(model.message_updated, self.emit_update)
 
-        self.connect(self, SIGNAL('about_to_update()'), self.save_selection,
-                     Qt.QueuedConnection)
-        self.connect(self, SIGNAL('update()'), self.update_actions,
-                     Qt.QueuedConnection)
+        self.about_to_update.connect(self.save_selection,
+                                     type=Qt.QueuedConnection)
+        self.updated.connect(self.update_actions, type=Qt.QueuedConnection)
 
-        # The non-Qt cola application model
-        self.connect(self, SIGNAL('expanded(QModelIndex)'), self.size_columns)
-        self.connect(self, SIGNAL('expanded(QModelIndex)'), self.expanded)
+        self.expanded.connect(self.index_expanded)
 
-        self.connect(self, SIGNAL('collapsed(QModelIndex)'), self.size_columns)
-        self.connect(self, SIGNAL('collapsed(QModelIndex)'), self.collapsed)
+        self.collapsed.connect(lambda idx: self.size_columns())
+        self.collapsed.connect(self.index_collapsed)
 
         # Sync selection before the key press event changes the model index
-        self.connect(self, SIGNAL('indexAboutToChange()'), self.sync_selection,
-                     Qt.QueuedConnection)
+        queued = Qt.QueuedConnection
+        self.index_about_to_change.connect(self.sync_selection, type=queued)
 
         self.action_history = qtutils.add_action_with_status_tip(
                 self, N_('View History...'),
-                N_('View history for selected path(s)'),
+                N_('View history for selected paths'),
                 self.view_history, hotkeys.HISTORY)
 
         self.action_stage = qtutils.add_action_with_status_tip(
                 self, cmds.StageOrUnstage.name(),
-                N_('Stage/unstage selected path(s) for commit'),
+                N_('Stage/unstage selected paths for commit'),
                 cmds.run(cmds.StageOrUnstage), hotkeys.STAGE_SELECTION)
 
         self.action_untrack = qtutils.add_action_with_status_tip(
                 self, N_('Untrack Selected'),
-                N_('Stop tracking path(s)'),
+                N_('Stop tracking paths'),
                 self.untrack_selected)
 
         self.action_difftool = qtutils.add_action_with_status_tip(
                 self, cmds.LaunchDifftool.name(),
-                N_('Launch git-difftool on the current path.'),
+                N_('Launch git-difftool on the current path'),
                 cmds.run(cmds.LaunchDifftool), hotkeys.DIFF)
 
         self.action_difftool_predecessor = qtutils.add_action_with_status_tip(
                 self, N_('Diff Against Predecessor...'),
-                N_('Launch git-difftool against previous versions.'),
-                self.difftool_predecessor, hotkeys.DIFF_SECONDARY)
+                N_('Launch git-difftool against previous versions'),
+                self.diff_predecessor, hotkeys.DIFF_SECONDARY)
 
         self.action_revert_unstaged = qtutils.add_action_with_status_tip(
                 self, cmds.RevertUnstagedEdits.name(),
-                N_('Revert unstaged changes to selected paths.'),
+                N_('Revert unstaged changes to selected paths'),
                 cmds.run(cmds.RevertUnstagedEdits), hotkeys.REVERT)
 
         self.action_revert_uncommitted = qtutils.add_action_with_status_tip(
                 self, cmds.RevertUncommittedEdits.name(),
-                N_('Revert uncommitted changes to selected paths.'),
+                N_('Revert uncommitted changes to selected paths'),
                 cmds.run(cmds.RevertUncommittedEdits), hotkeys.UNDO)
 
         self.action_editor = qtutils.add_action_with_status_tip(
                 self, cmds.LaunchEditor.name(),
-                N_('Edit selected path(s).'),
+                N_('Edit selected paths'),
                 cmds.run(cmds.LaunchEditor), hotkeys.EDIT)
 
-        self.action_refresh  = qtutils.add_action(
-                self, N_('Refresh'), cmds.run(cmds.Refresh), hotkeys.REFRESH)
+        self.action_refresh = common.refresh_action(self)
+
+        if not utils.is_win32():
+            self.action_default_app = common.default_app_action(
+                    self, self.selected_paths)
+
+            self.action_parent_dir = common.parent_dir_action(
+                    self, self.selected_paths)
+
+            self.action_terminal = common.terminal_action(
+                    self, self.selected_paths)
 
         self.x_width = QtGui.QFontMetrics(self.font()).width('x')
         self.size_columns()
 
-    def expanded(self, index):
-        item = self.model().itemFromIndex(index)
+    def index_expanded(self, index):
+        """Update information about a directory as it is expanded."""
+        # Remember open folders so that we can restore them when refreshing
+        item = self.item_from_index(index)
         self.saved_open_folders.add(item.path)
+        self.size_columns()
 
-    def collapsed(self, index):
-        item = self.model().itemFromIndex(index)
+        # update information about a directory as it is expanded
+        if item.cached:
+            return
+        path = item.path
+
+        model = self.model()
+        model.populate(item)
+        model.update_entry(path)
+
+        for row in range(item.rowCount()):
+            path = item.child(row, 0).path
+            model.update_entry(path)
+
+        item.cached = True
+
+    def index_collapsed(self, index):
+        item = self.item_from_index(index)
         self.saved_open_folders.remove(item.path)
 
     def refresh(self):
@@ -198,7 +242,7 @@ class RepoTreeView(standard.TreeView):
 
         if column == 1:
             # Status
-            size = x_width * 10
+            size = x_width * 11
         elif column == 2:
             # Summary
             size = x_width * 64
@@ -214,10 +258,10 @@ class RepoTreeView(standard.TreeView):
         return size
 
     def emit_update(self):
-        self.emit(SIGNAL('update()'))
+        self.updated.emit()
 
     def emit_about_to_update(self):
-        self.emit(SIGNAL('about_to_update()'))
+        self.about_to_update.emit()
 
     def save_selection(self):
         selection = self.selected_paths()
@@ -235,8 +279,9 @@ class RepoTreeView(standard.TreeView):
         self.restoring_selection = True
 
         # Restore opened folders
+        model = self.model()
         for path in sorted(self.saved_open_folders):
-            row = self.model().row(path, create=False)
+            row = model.get(path)
             if not row:
                 continue
             index = row[0].index()
@@ -248,7 +293,7 @@ class RepoTreeView(standard.TreeView):
         current_index = None
         current_path = self.saved_current_path
         if current_path:
-            row = self.model().row(current_path, create=False)
+            row = model.get(current_path)
             if row:
                 current_index = row[0].index()
 
@@ -257,7 +302,7 @@ class RepoTreeView(standard.TreeView):
 
         # Restore selected items
         for path in self.saved_selection:
-            row = self.model().row(path, create=False)
+            row = model.get(path)
             if not row:
                 continue
             index = row[0].index()
@@ -270,6 +315,21 @@ class RepoTreeView(standard.TreeView):
         self.size_columns()
         self.update_diff()
 
+    def event(self, ev):
+        """Respond to GitRepoInfoEvents"""
+        if ev.type() == self.info_event_type:
+            ev.accept()
+            self.apply_data(ev.data)
+        return super(RepoTreeView, self).event(ev)
+
+    def apply_data(self, data):
+        entry = self.model().get(data[0])
+        if entry:
+            entry[1].set_status(data[1])
+            entry[2].setText(data[2])
+            entry[3].setText(data[3])
+            entry[4].setText(data[4])
+
     def update_actions(self):
         """Enable/disable actions."""
         selection = self.selected_paths()
@@ -280,7 +340,13 @@ class RepoTreeView(standard.TreeView):
         tracked = bool(self.selected_tracked_paths(selection=selection))
         revertable = staged or modified
 
+        self.action_editor.setEnabled(selected)
         self.action_history.setEnabled(selected)
+        if not utils.is_win32():
+            self.action_default_app.setEnabled(selected)
+            self.action_parent_dir.setEnabled(selected)
+            self.action_terminal.setEnabled(selected)
+
         self.action_stage.setEnabled(staged or unstaged)
         self.action_untrack.setEnabled(tracked)
         self.action_difftool.setEnabled(staged or modified)
@@ -291,7 +357,7 @@ class RepoTreeView(standard.TreeView):
     def contextMenuEvent(self, event):
         """Create a context menu."""
         self.update_actions()
-        menu = QtGui.QMenu(self)
+        menu = qtutils.create_menu(N_('Actions'), self)
         menu.addAction(self.action_editor)
         menu.addAction(self.action_stage)
         menu.addSeparator()
@@ -302,11 +368,16 @@ class RepoTreeView(standard.TreeView):
         menu.addAction(self.action_revert_unstaged)
         menu.addAction(self.action_revert_uncommitted)
         menu.addAction(self.action_untrack)
+        if not utils.is_win32():
+            menu.addSeparator()
+            menu.addAction(self.action_default_app)
+            menu.addAction(self.action_parent_dir)
+            menu.addAction(self.action_terminal)
         menu.exec_(self.mapToGlobal(event.pos()))
 
     def mousePressEvent(self, event):
         """Synchronize the selection on mouse-press."""
-        result = QtGui.QTreeView.mousePressEvent(self, event)
+        result = QtWidgets.QTreeView.mousePressEvent(self, event)
         self.sync_selection()
         return result
 
@@ -342,7 +413,7 @@ class RepoTreeView(standard.TreeView):
 
     def selectionChanged(self, old, new):
         """Override selectionChanged to update available actions."""
-        result = QtGui.QTreeView.selectionChanged(self, old, new)
+        result = QtWidgets.QTreeView.selectionChanged(self, old, new)
         if not self.restoring_selection:
             self.update_actions()
             self.update_diff()
@@ -354,11 +425,10 @@ class RepoTreeView(standard.TreeView):
             cached = paths[0] in main.model().staged
             cmds.do(cmds.Diff, paths[0], cached)
 
-    def setModel(self, model):
+    def set_model(self, model):
         """Set the concrete QAbstractItemModel instance."""
-        QtGui.QTreeView.setModel(self, model)
-        self.connect(model, SIGNAL('restore()'), self.restore,
-                     Qt.QueuedConnection)
+        self.setModel(model)
+        model.restore.connect(self.restore, type=Qt.QueuedConnection)
 
     def item_from_index(self, model_index):
         """Return the name item corresponding to the model index."""
@@ -411,17 +481,25 @@ class RepoTreeView(standard.TreeView):
                 if p not in untracked or p in tracked]
 
     def view_history(self):
-        """Signal that we should view history for paths."""
-        self.emit(SIGNAL('history(PyQt_PyObject)'), self.selected_paths())
+        """Launch the configured history browser path-limited to entries."""
+        entries = list(map(ustr, entries))
+        cmds.do(cmds.VisualizePaths, entries)
 
     def untrack_selected(self):
         """untrack selected paths."""
         cmds.do(cmds.Untrack, self.selected_tracked_paths())
 
-    def difftool_predecessor(self):
+    def diff_predecessor(self):
         """Diff paths against previous versions."""
         paths = self.selected_tracked_paths()
-        self.emit(SIGNAL('difftool_predecessor(PyQt_PyObject)'), paths)
+        args = ['--'] + paths
+        revs, summaries = gitcmds.log_helper(all=False, extra_args=args)
+        commits = select_commits(N_('Select Previous Version'),
+                                 revs, summaries, multiselect=False)
+        if not commits:
+            return
+        commit = commits[0]
+        cmds.difftool_launch(left=commit, paths=paths)
 
     def current_path(self):
         """Return the path for the current item."""
@@ -431,56 +509,13 @@ class RepoTreeView(standard.TreeView):
         return self.item_from_index(index).path
 
 
-class BrowserController(QtCore.QObject):
-
-    def __init__(self, view=None):
-        QtCore.QObject.__init__(self, view)
-        self.model = main.model()
-        self.view = view
-        self.runtask = qtutils.RunTask(parent=self)
-        self.connect(view, SIGNAL('history(PyQt_PyObject)'),
-                     self.view_history)
-        self.connect(view, SIGNAL('expanded(QModelIndex)'),
-                     self.query_model)
-        self.connect(view, SIGNAL('difftool_predecessor(PyQt_PyObject)'),
-                     self.difftool_predecessor)
-
-    def view_history(self, entries):
-        """Launch the configured history browser path-limited to entries."""
-        entries = list(map(ustr, entries))
-        cmds.do(cmds.VisualizePaths, entries)
-
-    def query_model(self, model_index):
-        """Update information about a directory as it is expanded."""
-        item = self.view.item_from_index(model_index)
-        if item.cached:
-            return
-        path = item.path
-        GitRepoEntryStore.entry(path, self.view, self.runtask).update()
-        entry = GitRepoEntryStore.entry
-        for row in range(item.rowCount()):
-            path = item.child(row, 0).path
-            entry(path, self.view, self.runtask).update()
-        item.cached = True
-
-    def difftool_predecessor(self, paths):
-        """Prompt for an older commit and launch difftool against it."""
-        args = ['--'] + paths
-        revs, summaries = gitcmds.log_helper(all=False, extra_args=args)
-        commits = select_commits(N_('Select Previous Version'),
-                                 revs, summaries, multiselect=False)
-        if not commits:
-            return
-        commit = commits[0]
-        difftool.launch(left=commit, paths=paths)
-
-
 class BrowseModel(object):
+    """Context data used for browsing branches via git-ls-tree"""
 
-    def __init__(self, ref):
+    def __init__(self, ref, filename=None):
         self.ref = ref
-        self.relpath = None
-        self.filename = None
+        self.relpath = filename
+        self.filename = filename
 
 
 class SaveBlob(BaseCommand):
@@ -508,53 +543,45 @@ class SaveBlob(BaseCommand):
                 N_('File saved to "%s"') % model.filename)
 
 
+class BrowseBranch(standard.Dialog):
 
-class BrowseDialog(QtGui.QDialog):
-
-    @staticmethod
-    def browse(ref):
-        parent = qtutils.active_window()
+    @classmethod
+    def browse(cls, ref):
         model = BrowseModel(ref)
-        dlg = BrowseDialog(model, parent=parent)
+        dlg = cls(model, parent=qtutils.active_window())
         dlg_model = GitTreeModel(ref, dlg)
         dlg.setModel(dlg_model)
         dlg.setWindowTitle(N_('Browsing %s') % model.ref)
-        if hasattr(parent, 'width'):
-            dlg.resize(parent.width()*3//4, 333)
-        else:
-            dlg.resize(420, 333)
         dlg.show()
         dlg.raise_()
         if dlg.exec_() != dlg.Accepted:
             return None
         return dlg
 
-    @staticmethod
-    def select_file(ref):
+    @classmethod
+    def select_file(cls, ref):
         parent = qtutils.active_window()
         model = BrowseModel(ref)
-        dlg = BrowseDialog(model, select_file=True, parent=parent)
+        dlg = cls(model, select_file=True, parent=parent)
         dlg_model = GitTreeModel(ref, dlg)
         dlg.setModel(dlg_model)
         dlg.setWindowTitle(N_('Select file from "%s"') % model.ref)
-        dlg.resize(parent.width()*3//4, 333)
         dlg.show()
         dlg.raise_()
         if dlg.exec_() != dlg.Accepted:
             return None
         return model.filename
 
-    @staticmethod
-    def select_file_from_list(file_list, title=N_('Select File')):
+    @classmethod
+    def select_file_from_list(cls, file_list, title=N_('Select File')):
         parent = qtutils.active_window()
         model = BrowseModel(None)
-        dlg = BrowseDialog(model, select_file=True, parent=parent)
+        dlg = cls(model, select_file=True, parent=parent)
         dlg_model = GitFileTreeModel(dlg)
         dlg_model.add_files(file_list)
         dlg.setModel(dlg_model)
         dlg.expandAll()
         dlg.setWindowTitle(title)
-        dlg.resize(parent.width()*3//4, 333)
         dlg.show()
         dlg.raise_()
         if dlg.exec_() != dlg.Accepted:
@@ -562,8 +589,7 @@ class BrowseDialog(QtGui.QDialog):
         return model.filename
 
     def __init__(self, model, select_file=False, parent=None):
-        QtGui.QDialog.__init__(self, parent)
-        self.setAttribute(Qt.WA_MacMetalStyle)
+        standard.Dialog.__init__(self, parent=parent)
         if parent is not None:
             self.setWindowModality(Qt.WindowModal)
 
@@ -572,7 +598,7 @@ class BrowseDialog(QtGui.QDialog):
 
         # widgets
         self.tree = GitTreeWidget(parent=self)
-        self.close = qtutils.close_button()
+        self.close_button = qtutils.close_button()
 
         if select_file:
             text = N_('Select')
@@ -583,7 +609,8 @@ class BrowseDialog(QtGui.QDialog):
 
         # layouts
         self.btnlayt = qtutils.hbox(defs.margin, defs.spacing,
-                                    qtutils.STRETCH, self.close, self.save)
+                                    self.close_button, qtutils.STRETCH,
+                                    self.save)
 
         self.layt = qtutils.vbox(defs.margin, defs.spacing,
                                  self.tree, self.btnlayt)
@@ -591,17 +618,16 @@ class BrowseDialog(QtGui.QDialog):
 
         # connections
         if select_file:
-            self.connect(self.tree, SIGNAL('path_chosen(PyQt_PyObject)'),
-                         self.path_chosen)
+            self.tree.path_chosen.connect(self.path_chosen)
         else:
-            self.connect(self.tree, SIGNAL('path_chosen(PyQt_PyObject)'),
-                         self.save_path)
+            self.tree.path_chosen.connect(self.save_path)
 
-        self.connect(self.tree, SIGNAL('selectionChanged()'),
-                     self.selection_changed, Qt.QueuedConnection)
+        self.tree.selection_changed.connect(self.selection_changed,
+                                            type=Qt.QueuedConnection)
 
-        qtutils.connect_button(self.close, self.reject)
+        qtutils.connect_button(self.close_button, self.close)
         qtutils.connect_button(self.save, self.save_blob)
+        self.init_size(parent=parent)
 
     def expandAll(self):
         self.tree.expandAll()
@@ -620,20 +646,15 @@ class BrowseDialog(QtGui.QDialog):
     def save_path(self, path):
         """Choose an output filename based on the selected path"""
         self.path_chosen(path, close=False)
-        model = self.model
-        filename = qtutils.save_as(model.filename)
-        if not filename:
-            return
-        model.filename = filename
-        cmds.do(SaveBlob, model)
-        self.accept()
+        if save_path(path, self.model):
+            self.accept()
 
     def save_blob(self):
         """Save the currently selected file"""
         filenames = self.tree.selected_files()
         if not filenames:
             return
-        self.path_chosen(filenames[0], close=True)
+        self.save_path(filenames[0])
 
     def selection_changed(self):
         """Update actions based on the current selection"""
@@ -643,12 +664,13 @@ class BrowseDialog(QtGui.QDialog):
 
 class GitTreeWidget(standard.TreeView):
 
+    selection_changed = Signal()
+    path_chosen = Signal(object)
+
     def __init__(self, parent=None):
         standard.TreeView.__init__(self, parent)
         self.setHeaderHidden(True)
-
-        self.connect(self, SIGNAL('doubleClicked(QModelIndex)'),
-                     self.double_clicked)
+        self.doubleClicked.connect(self.double_clicked)
 
     def double_clicked(self, index):
         item = self.model().itemFromIndex(index)
@@ -656,15 +678,15 @@ class GitTreeWidget(standard.TreeView):
             return
         if item.is_dir:
             return
-        self.emit(SIGNAL('path_chosen(PyQt_PyObject)'), item.path)
+        self.path_chosen.emit(item.path)
 
     def selected_files(self):
         items = self.selected_items()
         return [i.path for i in items if not i.is_dir]
 
     def selectionChanged(self, old_selection, new_selection):
-        QtGui.QTreeView.selectionChanged(self, old_selection, new_selection)
-        self.emit(SIGNAL('selectionChanged()'))
+        QtWidgets.QTreeView.selectionChanged(self, old_selection, new_selection)
+        self.selection_changed.emit()
 
     def select_first_file(self):
         """Select the first filename in the tree"""
