@@ -1,17 +1,16 @@
-from PyQt4 import QtCore
-from PyQt4 import QtGui
+from __future__ import absolute_import, division, unicode_literals
 
-from cola import decorators
-from cola import qtutils
-from cola.compat import ustr
-from cola.widgets import standard
+from qtpy import QtCore
+from qtpy import QtWidgets
+
+from .. import icons
+from . import standard
 
 
 class FileTree(standard.TreeWidget):
 
     def __init__(self, parent=None):
         standard.TreeWidget.__init__(self, parent=parent)
-        self.icon_from_filename = decorators.memoize(qtutils.icon_from_filename)
         self.setSelectionMode(self.ExtendedSelection)
         self.setHeaderHidden(True)
 
@@ -20,19 +19,20 @@ class FileTree(standard.TreeWidget):
         if not filenames:
             return
         items = []
+        from_filename = icons.from_filename
         for filename in filenames:
-            icon = self.icon_from_filename(filename)
-            item = QtGui.QTreeWidgetItem()
+            icon = from_filename(filename)
+            item = QtWidgets.QTreeWidgetItem()
             item.setIcon(0, icon)
             item.setText(0, filename)
-            item.setData(0, QtCore.Qt.UserRole, QtCore.QVariant(filename))
+            item.setData(0, QtCore.Qt.UserRole, filename)
             items.append(item)
         self.addTopLevelItems(items)
-        if select:
-            self.setItemSelected(items[0], True)
+        if select and items:
+            items[0].setSelected(True)
 
     def filename_from_item(self, item):
-        return ustr(item.data(0, QtCore.Qt.UserRole).toPyObject())
+        return item.data(0, QtCore.Qt.UserRole)
 
     def has_selection(self):
         return bool(self.selectedItems())
