@@ -4,61 +4,67 @@ from __future__ import division, absolute_import, unicode_literals
 
 import os
 
-from PyQt4 import QtGui
-from PyQt4.QtCore import Qt
-from PyQt4.QtCore import SIGNAL
+from qtpy import QtCore
+from qtpy import QtGui
+from qtpy import QtWidgets
+from qtpy.QtCore import Qt
+from qtpy.QtCore import Signal
 
-from cola import cmds
-from cola import core
-from cola import guicmds
-from cola import git
-from cola import gitcfg
-from cola import hotkeys
-from cola import icons
-from cola import qtutils
-from cola import resources
-from cola import utils
-from cola import version
-from cola.compat import unichr
-from cola.i18n import N_
-from cola.interaction import Interaction
-from cola.models import prefs
-from cola.settings import Settings
-from cola.widgets import about
-from cola.widgets import action
-from cola.widgets import archive
-from cola.widgets import bookmarks
-from cola.widgets import browse
-from cola.widgets import cfgactions
-from cola.widgets import commitmsg
-from cola.widgets import compare
-from cola.widgets import createbranch
-from cola.widgets import createtag
-from cola.widgets import dag
-from cola.widgets import defs
-from cola.widgets import diff
-from cola.widgets import finder
-from cola.widgets import editremotes
-from cola.widgets import grep
-from cola.widgets import log
-from cola.widgets import merge
-from cola.widgets import patch
-from cola.widgets import prefs as prefs_widget
-from cola.widgets import recent
-from cola.widgets import remote
-from cola.widgets import search
-from cola.widgets import standard
-from cola.widgets import status
-from cola.widgets import stash
+from ..compat import unichr
+from ..i18n import N_
+from ..interaction import Interaction
+from ..models import prefs
+from ..settings import Settings
+from .. import cmds
+from .. import core
+from .. import guicmds
+from .. import git
+from .. import gitcfg
+from .. import gitcmds
+from .. import hotkeys
+from .. import icons
+from .. import qtutils
+from .. import resources
+from .. import utils
+from .. import version
+from . import about
+from . import action
+from . import archive
+from . import bookmarks
+from . import branch
+from . import browse
+from . import cfgactions
+from . import commitmsg
+from . import compare
+from . import createbranch
+from . import createtag
+from . import dag
+from . import defs
+from . import diff
+from . import finder
+from . import editremotes
+from . import grep
+from . import log
+from . import merge
+from . import patch
+from . import prefs as prefs_widget
+from . import recent
+from . import remote
+from . import search
+from . import standard
+from . import status
+from . import stash
 
 
 class MainView(standard.MainWindow):
+    config_actions_changed = Signal(object)
+    updated = Signal()
 
     def __init__(self, model, parent=None, settings=None):
         standard.MainWindow.__init__(self, parent)
-        self.setAttribute(Qt.WA_MacMetalStyle)
 
         # Default size; this is thrown out when save/restore is used
+        self.dag = None
         self.model = model
         self.settings = settings
         self.prefs_model = prefs_model = prefs.PreferencesModel()
@@ -69,7 +75,6 @@ class MainView(standard.MainWindow):
 
         # Runs asynchronous tasks
         self.runtask = qtutils.RunTask()
-        self.progress = standard.ProgressDialog('', '', self)
 
         create_dock = qtutils.create_dock
         cfg = gitcfg.current()
@@ -77,7 +82,8 @@ class MainView(standard.MainWindow):
                                  cfg.get('cola.classicdockable'))
         if self.browser_dockable:
             self.browserdockwidget = create_dock(N_('Browser'), self)
-            self.browserwidget = browse.worktree_browser_widget(self)
+            self.browserwidget = (
+                    browse.worktree_browser(parent=self, update=False))
             self.browserdockwidget.setWidget(self.browserwidget)
 
         # "Actions" widget
@@ -105,9 +111,15 @@ class MainView(standard.MainWindow):
                 bookmarks.RECENT_REPOS, parent=self.recentdockwidget)
         self.recentdockwidget.setWidget(self.recentwidget)
         self.recentdockwidget.hide()
+        self.bookmarkswidget.connect_to(self.recentwidget)
+
+        # "Branch" widgets
+        self.branchdockwidget = create_dock(N_('Branches'), self)
+        self.branchwidget = branch.BranchesWidget(parent=self.branchdockwidget)
+        self.branchdockwidget.setWidget(self.branchwidget)
 
         # "Commit Message Editor" widget
-        self.position_label = QtGui.QLabel()
+        self.position_label = QtWidgets.QLabel()
         self.position_label.setAlignment(Qt.AlignCenter)
         font = qtutils.default_monospace_font()
         font.setPointSize(int(font.pointSize() * 0.8))
@@ -140,6 +152,13 @@ class MainView(standard.MainWindow):
 
         # All Actions
         add_action = qtutils.add_action
+        add_action_bool = qtutils.add_action_bool
+
+        self.commit_amend_action = add_action_bool(
+            self, N_('Amend Last Commit'), cmds.run(cmds.AmendMode), False)
+        self.commit_amend_action.setShortcut(hotkeys.AMEND)
+        self.commit_amend_action.setShortcutContext(Qt.WidgetShortcut)
+
         self.unstage_all_action = add_action(
             self, N_('Unstage All'), cmds.run(cmds.UnstageAll))
         self.unstage_all_action.setIcon(icons.remove())
@@ -201,13 +220,19 @@ class MainView(standard.MainWindow):
         self.load_commitmsg_action = add_action(
             self, N_('Load Commit Message...'), guicmds.load_commitmsg)
 
+        self.prepare_commitmsg_hook_action = add_action(
+            self, N_('Prepare Commit Message'),
+            cmds.run(cmds.PrepareCommitMessageHook))
+
         self.save_tarball_action = add_action(
             self, N_('Save As Tarball/Zip...'), self.save_archive)
 
         self.quit_action = add_action(
             self, N_('Quit'), self.close, hotkeys.QUIT)
+
         self.grep_action = add_action(
             self, N_('Grep'), grep.grep, hotkeys.GREP)
+
         self.merge_local_action = add_action(
             self, N_('Merge...'), merge.local_merge, hotkeys.MERGE)
 
@@ -222,7 +247,7 @@ class MainView(standard.MainWindow):
             self, N_('Pull...'), remote.pull, hotkeys.PULL)
 
         self.open_repo_action = add_action(
-            self, N_('Open...'), guicmds.open_repo)
+            self, N_('Open...'), guicmds.open_repo, hotkeys.OPEN)
         self.open_repo_action.setIcon(icons.folder())
 
         self.open_repo_new_action = add_action(
@@ -232,6 +257,12 @@ class MainView(standard.MainWindow):
         self.stash_action = add_action(
             self, N_('Stash...'), stash.stash, hotkeys.STASH)
 
+        self.reset_branch_head_action = add_action(
+            self, N_('Reset Branch Head'), guicmds.reset_branch_head)
+
+        self.reset_worktree_action = add_action(
+            self, N_('Reset Worktree'), guicmds.reset_worktree)
+
         self.clone_repo_action = add_action(
             self, N_('Clone...'), self.clone_repo)
         self.clone_repo_action.setIcon(icons.repo())
@@ -252,6 +283,7 @@ class MainView(standard.MainWindow):
             cmds.run(cmds.VisualizeAll))
         self.search_commits_action = add_action(
             self, N_('Search...'), search.search)
+
         self.browse_branch_action = add_action(
             self, N_('Browse Current Branch...'), guicmds.browse_current)
         self.browse_other_branch_action = add_action(
@@ -260,7 +292,7 @@ class MainView(standard.MainWindow):
             self, N_('Get Commit Message Template'),
             cmds.run(cmds.LoadCommitMessageFromTemplate))
         self.help_about_action = add_action(
-            self, N_('About'), about.launch_about_dialog)
+            self, N_('About'), about.about_dialog)
 
         self.diff_expression_action = add_action(
             self, N_('Expression...'), guicmds.diff_expression)
@@ -291,7 +323,8 @@ class MainView(standard.MainWindow):
             self, N_('Review...'), guicmds.review_branch)
 
         self.browse_action = add_action(
-            self, N_('File Browser...'), browse.worktree_browser)
+            self, N_('File Browser...'),
+            lambda: browse.worktree_browser(show=True))
         self.browse_action.setIcon(icons.cola())
 
         self.dag_action = add_action(self, N_('DAG...'), self.git_dag)
@@ -315,9 +348,9 @@ class MainView(standard.MainWindow):
         # For "Start Rebase" only, reverse the first argument to setEnabled()
         # so that we can operate on it as a group.
         # We can do this because can_rebase == not is_rebasing
-        set_disabled = lambda x: self.rebase_start_action.setEnabled(not x)
-        self.rebase_start_action_proxy = utils.Proxy(self.rebase_start_action,
-                                                     setEnabled=set_disabled)
+        self.rebase_start_action_proxy = utils.Proxy(
+                self.rebase_start_action,
+                setEnabled=lambda x: self.rebase_start_action.setEnabled(not x))
 
         self.rebase_group = utils.Group(self.rebase_start_action_proxy,
                                         self.rebase_edit_todo_action,
@@ -325,11 +358,11 @@ class MainView(standard.MainWindow):
                                         self.rebase_skip_action,
                                         self.rebase_abort_action)
 
-        self.lock_layout_action = qtutils.add_action_bool(
+        self.lock_layout_action = add_action_bool(
             self, N_('Lock Layout'), self.set_lock_layout, False)
 
         # Create the application menu
-        self.menubar = QtGui.QMenuBar(self)
+        self.menubar = QtWidgets.QMenuBar(self)
 
         # File Menu
         create_menu = qtutils.create_menu
@@ -346,9 +379,6 @@ class MainView(standard.MainWindow):
         self.file_menu.addAction(self.edit_remotes_action)
         self.file_menu.addAction(self.browse_recently_modified_action)
         self.file_menu.addSeparator()
-        self.file_menu.addAction(self.load_commitmsg_action)
-        self.file_menu.addAction(self.load_commitmsg_template_action)
-        self.file_menu.addSeparator()
         self.file_menu.addAction(self.apply_patches_action)
         self.file_menu.addAction(self.export_patches_action)
         self.file_menu.addAction(self.save_tarball_action)
@@ -369,18 +399,28 @@ class MainView(standard.MainWindow):
         self.actions_menu.addAction(self.merge_local_action)
         self.actions_menu.addAction(self.merge_abort_action)
         self.actions_menu.addSeparator()
+        self.actions_reset_menu = self.actions_menu.addMenu(N_('Reset'))
+        self.actions_reset_menu.addAction(self.reset_branch_head_action)
+        self.actions_reset_menu.addAction(self.reset_worktree_action)
+        self.actions_menu.addSeparator()
         self.actions_menu.addAction(self.grep_action)
         self.actions_menu.addAction(self.search_commits_action)
         self.menubar.addAction(self.actions_menu.menuAction())
 
-        # Staging Area Menu
-        self.commit_menu = create_menu(N_('Staging Area'), self.menubar)
-        self.commit_menu.setTitle(N_('Staging Area'))
+        # Commit Menu
+        self.commit_menu = create_menu(N_('Commit@@verb'), self.menubar)
+        self.commit_menu.setTitle(N_('Commit@@verb'))
+        self.commit_menu.addAction(self.commit_amend_action)
+        self.commit_menu.addSeparator()
         self.commit_menu.addAction(self.stage_modified_action)
         self.commit_menu.addAction(self.stage_untracked_action)
         self.commit_menu.addSeparator()
         self.commit_menu.addAction(self.unstage_all_action)
         self.commit_menu.addAction(self.unstage_selected_action)
+        self.commit_menu.addSeparator()
+        self.commit_menu.addAction(self.load_commitmsg_action)
+        self.commit_menu.addAction(self.load_commitmsg_template_action)
+        self.commit_menu.addAction(self.prepare_commitmsg_hook_action)
         self.menubar.addAction(self.commit_menu.menuAction())
 
         # Diff Menu
@@ -454,15 +494,16 @@ class MainView(standard.MainWindow):
         self.addDockWidget(left, self.diffdockwidget)
         self.addDockWidget(right, self.statusdockwidget)
         self.addDockWidget(right, self.bookmarksdockwidget)
+        self.addDockWidget(right, self.branchdockwidget)
         self.addDockWidget(right, self.recentdockwidget)
         self.addDockWidget(bottom, self.actionsdockwidget)
         self.addDockWidget(bottom, self.logdockwidget)
         self.tabifyDockWidget(self.actionsdockwidget, self.logdockwidget)
 
-
         # Listen for model notifications
-        model.add_observer(model.message_updated, self._update)
-        model.add_observer(model.message_mode_changed, lambda x: self._update())
+        model.add_observer(model.message_updated, self.updated.emit)
+        model.add_observer(model.message_mode_changed,
+                           lambda mode: self.updated.emit())
 
         prefs_model.add_observer(prefs_model.message_config_updated,
                                  self._config_updated)
@@ -470,50 +511,36 @@ class MainView(standard.MainWindow):
         # Set a default value
         self.show_cursor_position(1, 0)
 
-        self.connect(self.open_recent_menu, SIGNAL('aboutToShow()'),
-                     self.build_recent_menu)
+        self.commit_menu.aboutToShow.connect(self.update_menu_actions)
+        self.open_recent_menu.aboutToShow.connect(self.build_recent_menu)
+        self.commitmsgeditor.cursor_changed.connect(self.show_cursor_position)
 
-        self.connect(self.commitmsgeditor, SIGNAL('cursorPosition(int,int)'),
-                     self.show_cursor_position)
+        self.diffeditor.options_changed.connect(self.statuswidget.refresh)
+        self.diffeditor.up.connect(self.statuswidget.move_up)
+        self.diffeditor.down.connect(self.statuswidget.move_down)
 
-        self.connect(self.diffeditor, SIGNAL('diff_options_updated()'),
-                     self.statuswidget.refresh)
+        self.commitmsgeditor.up.connect(self.statuswidget.move_up)
+        self.commitmsgeditor.down.connect(self.statuswidget.move_down)
 
-        self.connect(self.diffeditor, SIGNAL('move_down()'),
-                     self.statuswidget.move_down)
-
-        self.connect(self.diffeditor, SIGNAL('move_up()'),
-                     self.statuswidget.move_up)
-
-        self.connect(self.commitmsgeditor, SIGNAL('move_down()'),
-                     self.statuswidget.move_down)
-
-        self.connect(self.commitmsgeditor, SIGNAL('move_up()'),
-                     self.statuswidget.move_up)
-
-        self.connect(self, SIGNAL('update()'),
-                     self._update_callback, Qt.QueuedConnection)
-
-        self.connect(self, SIGNAL('install_cfg_actions(PyQt_PyObject)'),
-                     self._install_config_actions, Qt.QueuedConnection)
+        self.updated.connect(self.refresh, type=Qt.QueuedConnection)
 
+        self.config_actions_changed.connect(self._install_config_actions,
+                                            type=Qt.QueuedConnection)
         # Install .git-config-defined actions
         self.init_config_actions()
-
-        # Restore saved settings
-        if not self.restore_state(settings=settings):
-            self.resize(987, 610)
-            self.set_initial_size()
-
-        self.statusdockwidget.widget().setFocus()
+        self.init_state(settings, self.set_initial_size)
 
         # Route command output here
         Interaction.log_status = self.logwidget.log_status
         Interaction.log = self.logwidget.log
+        Interaction.safe_log = self.logwidget.safe_log
         Interaction.log(version.git_version_str() + '\n' +
                         N_('git cola version %s') % version.version())
+        # Focus the status widget; this must be deferred
+        QtCore.QTimer.singleShot(0, lambda: self.statuswidget.setFocus())
 
     def set_initial_size(self):
+        self.resize(987, 610)
         self.statuswidget.set_initial_size()
         self.commitmsgeditor.set_initial_size()
 
@@ -524,7 +551,7 @@ class MainView(standard.MainWindow):
     def closeEvent(self, event):
         """Save state in the settings manager."""
         commit_msg = self.commitmsgeditor.commit_message(raw=True)
-        self.model.save_commitmsg(commit_msg)
+        self.model.save_commitmsg(msg=commit_msg)
         standard.MainWindow.closeEvent(self, event)
 
     def build_recent_menu(self):
@@ -535,10 +562,10 @@ class MainView(standard.MainWindow):
         menu = self.open_recent_menu
         menu.clear()
         for r in recent:
-            name = os.path.basename(r)
-            directory = os.path.dirname(r)
+            name = r['name']
+            directory = r['path']
             text = '%s %s %s' % (name, unichr(0x2192), directory)
-            menu.addAction(text, cmds.run(cmd, r))
+            menu.addAction(text, cmds.run(cmd, directory))
 
     # Accessors
     mode = property(lambda self: self.model.mode)
@@ -576,7 +603,7 @@ class MainView(standard.MainWindow):
 
     def get_config_actions(self):
         actions = cfgactions.get_config_actions()
-        self.emit(SIGNAL('install_cfg_actions(PyQt_PyObject)'), actions)
+        self.config_actions_changed.emit(actions)
 
     def _install_config_actions(self, names_and_shortcuts):
         """Install .gitconfig-defined actions"""
@@ -589,10 +616,7 @@ class MainView(standard.MainWindow):
             if shortcut:
                 action.setShortcut(shortcut)
 
-    def _update(self):
-        self.emit(SIGNAL('update()'))
-
-    def _update_callback(self):
+    def refresh(self):
         """Update the title with the current branch and directory name."""
         alerts = []
         branch = self.model.currentbranch
@@ -619,6 +643,9 @@ class MainView(standard.MainWindow):
 
         if self.mode == self.model.mode_amend:
             alerts.append(N_('Amending'))
+            self.commit_amend_action.setChecked(True)
+        else:
+            self.commit_amend_action.setChecked(False)
 
         l = unichr(0xab)
         r = unichr(0xbb)
@@ -641,10 +668,17 @@ class MainView(standard.MainWindow):
         self.rename_branch_action.setEnabled(enabled)
         self.delete_branch_action.setEnabled(enabled)
 
+    def update_menu_actions(self):
+        # Enable the Prepare Commit Message action if the hook exists
+        hook = gitcmds.prepare_commit_message_hook()
+        enabled = os.path.exists(hook)
+        self.prepare_commitmsg_hook_action.setEnabled(enabled)
+
     def export_state(self):
         state = standard.MainWindow.export_state(self)
         show_status_filter = self.statuswidget.filter_widget.isVisible()
         state['show_status_filter'] = show_status_filter
+        state['show_diff_line_numbers'] = self.diffeditor.show_line_numbers()
         return state
 
     def apply_state(self, state):
@@ -654,6 +688,10 @@ class MainView(standard.MainWindow):
 
         show_status_filter = state.get('show_status_filter', False)
         self.statuswidget.filter_widget.setVisible(show_status_filter)
+
+        diff_numbers = state.get('show_diff_line_numbers', False)
+        self.diffeditor.enable_line_numbers(diff_numbers)
+
         return result
 
     def setup_dockwidget_view_menu(self):
@@ -670,44 +708,58 @@ class MainView(standard.MainWindow):
             (optkey + '+4', self.actionsdockwidget),
             (optkey + '+5', self.bookmarksdockwidget),
             (optkey + '+6', self.recentdockwidget),
+            (optkey + '+7', self.branchdockwidget),
         )
         for shortcut, dockwidget in dockwidgets:
             # Associate the action with the shortcut
             toggleview = dockwidget.toggleViewAction()
             toggleview.setShortcut('Shift+' + shortcut)
             self.view_menu.addAction(toggleview)
+
             def showdock(show, dockwidget=dockwidget):
                 if show:
                     dockwidget.raise_()
                     dockwidget.widget().setFocus()
                 else:
                     self.setFocus()
+
             self.addAction(toggleview)
             qtutils.connect_action_bool(toggleview, showdock)
 
             # Create a new shortcut Shift+<shortcut> that gives focus
-            toggleview = QtGui.QAction(self)
+            toggleview = QtWidgets.QAction(self)
             toggleview.setShortcut(shortcut)
+
             def focusdock(dockwidget=dockwidget):
                 focus_dock(dockwidget)
             self.addAction(toggleview)
             qtutils.connect_action(toggleview, focusdock)
 
-        focus = lambda: focus_dock(self.commitdockwidget)
-        qtutils.add_action(self, 'Focus Commit Message', focus, hotkeys.FOCUS)
+        # These widgets warrant home-row hotkey status
+        qtutils.add_action(self, 'Focus Commit Message',
+                           lambda: focus_dock(self.commitdockwidget),
+                           hotkeys.FOCUS)
+
+        qtutils.add_action(self, 'Focus Status Window',
+                           lambda: focus_dock(self.statusdockwidget),
+                           hotkeys.FOCUS_STATUS)
+
+        qtutils.add_action(self, 'Focus Diff Editor',
+                           lambda: focus_dock(self.diffdockwidget),
+                           hotkeys.FOCUS_DIFF)
 
     def preferences(self):
         return prefs_widget.preferences(model=self.prefs_model, parent=self)
 
     def git_dag(self):
-        view = dag.git_dag(self.model)
+        self.dag = dag.git_dag(self.model, existing_view=self.dag)
+        view = self.dag
         view.show()
         view.raise_()
 
     def save_archive(self):
-        ref = self.model.git.rev_parse('HEAD')[git.STDOUT]
-        shortref = ref[:7]
-        archive.GitArchiveDialog.save_hashed_objects(ref, shortref, self)
+        oid = self.model.git.rev_parse('HEAD')[git.STDOUT]
+        archive.show_save_dialog(oid, parent=self)
 
     def show_cursor_position(self, rows, cols):
         display = '%02d:%02d' % (rows, cols)
@@ -755,7 +807,7 @@ class MainView(standard.MainWindow):
         if not upstream:
             return
         self.model.is_rebasing = True
-        self._update_callback()
+        self.refresh()
         cmds.do(cmds.Rebase, upstream=upstream)
 
     def rebase_edit_todo(self):
@@ -771,7 +823,8 @@ class MainView(standard.MainWindow):
         cmds.do(cmds.RebaseAbort)
 
     def clone_repo(self):
-        guicmds.clone_repo(self, self.runtask, self.progress,
+        progress = standard.ProgressDialog('', '', self)
+        guicmds.clone_repo(self, self.runtask, progress,
                            guicmds.report_clone_repo_errors, True)
 
 
