@@ -1,27 +1,28 @@
 from __future__ import division, absolute_import, unicode_literals
-
 import itertools
 
-from PyQt4 import QtCore
-from PyQt4 import QtGui
-from PyQt4.QtCore import Qt
-from PyQt4.QtCore import SIGNAL
+from qtpy.QtCore import Qt
+from qtpy.QtCore import Signal
+from qtpy import QtCore
+from qtpy import QtWidgets
 
-from cola import cmds
-from cola import core
-from cola import hotkeys
-from cola import icons
-from cola import qtutils
-from cola import utils
-from cola.i18n import N_
-from cola.models import main
-from cola.models import prefs
-from cola.models import selection
-from cola.widgets import completion
-from cola.widgets import defs
+from ..i18n import N_
+from ..models import main
+from ..models import prefs
+from ..models import selection
+from ..widgets import gitignore
+from .. import cmds
+from .. import core
+from .. import hotkeys
+from .. import icons
+from .. import qtutils
+from .. import utils
+from . import common
+from . import completion
+from . import defs
 
 
-class StatusWidget(QtGui.QWidget):
+class StatusWidget(QtWidgets.QWidget):
     """
     Provides a git-status-like repository widget.
 
@@ -29,8 +30,9 @@ class StatusWidget(QtGui.QWidget):
     Qt signals.
 
     """
+
     def __init__(self, titlebar, parent=None):
-        QtGui.QWidget.__init__(self, parent)
+        QtWidgets.QWidget.__init__(self, parent)
 
         tooltip = N_('Toggle the paths filter')
         icon = icons.ellipsis()
@@ -82,7 +84,11 @@ class StatusWidget(QtGui.QWidget):
         self.tree.move_down()
 
 
-class StatusTreeWidget(QtGui.QTreeWidget):
+class StatusTreeWidget(QtWidgets.QTreeWidget):
+    # Signals
+    about_to_update = Signal()
+    updated = Signal()
+
     # Item categories
     idx_header = -1
     idx_staged = 0
@@ -95,9 +101,9 @@ class StatusTreeWidget(QtGui.QTreeWidget):
     mode = property(lambda self: self.m.mode)
 
     def __init__(self, parent=None):
-        QtGui.QTreeWidget.__init__(self, parent)
+        QtWidgets.QTreeWidget.__init__(self, parent)
 
-        self.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
+        self.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
         self.headerItem().setHidden(True)
         self.setAllColumnsShowFocus(True)
         self.setSortingEnabled(False)
@@ -144,15 +150,14 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         self.launch_editor_action.setIcon(icons.edit())
 
         if not utils.is_win32():
-            self.open_using_default_app = qtutils.add_action(
-                self, cmds.OpenDefaultApp.name(),
-                self._open_using_default_app, hotkeys.PRIMARY_ACTION)
-            self.open_using_default_app.setIcon(icons.default_app())
+            self.default_app_action = common.default_app_action(
+                    self, self.selected_group)
 
-            self.open_parent_dir_action = qtutils.add_action(
-                self, cmds.OpenParentDir.name(),
-                self._open_parent_dir, hotkeys.SECONDARY_ACTION)
-            self.open_parent_dir_action.setIcon(icons.folder())
+            self.parent_dir_action = common.parent_dir_action(
+                    self, self.selected_group)
+
+            self.terminal_action = common.terminal_action(
+                    self, self.selected_group)
 
         self.up_action = qtutils.add_action(
             self, N_('Move Up'), self.move_up,
@@ -174,13 +179,16 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         self.view_history_action = qtutils.add_action(
             self, N_('View History...'), self.view_history, hotkeys.HISTORY)
 
+        self.view_blame_action = qtutils.add_action(
+            self, N_('Blame...'), self.view_blame, hotkeys.BLAME)
+
         # MoveToTrash and Delete use the same shortcut.
         # We will only bind one of them, depending on whether or not the
-        # MoveToTrash command is avaialble.  When available, the hotkey
+        # MoveToTrash command is available.  When available, the hotkey
         # is bound to MoveToTrash, otherwise it is bound to Delete.
         if cmds.MoveToTrash.AVAILABLE:
             self.move_to_trash_action = qtutils.add_action(
-                self, N_('Move file(s) to trash'),
+                self, N_('Move files to trash'),
                 self._trash_untracked_files, hotkeys.TRASH)
             self.move_to_trash_action.setIcon(icons.discard())
             delete_shortcut = hotkeys.DELETE_FILE
@@ -189,31 +197,23 @@ class StatusTreeWidget(QtGui.QTreeWidget):
             delete_shortcut = hotkeys.DELETE_FILE_SECONDARY
 
         self.delete_untracked_files_action = qtutils.add_action(
-            self, N_('Delete File(s)...'),
+            self, N_('Delete Files...'),
             self._delete_untracked_files, delete_shortcut)
         self.delete_untracked_files_action.setIcon(icons.discard())
 
-        self.connect(self, SIGNAL('about_to_update()'),
-                     self._about_to_update, Qt.QueuedConnection)
-        self.connect(self, SIGNAL('updated()'),
-                     self._updated, Qt.QueuedConnection)
+        about_to_update = self._about_to_update
+        self.about_to_update.connect(about_to_update, type=Qt.QueuedConnection)
+        self.updated.connect(self.refresh, type=Qt.QueuedConnection)
 
         self.m = main.model()
         self.m.add_observer(self.m.message_about_to_update,
-                            self.about_to_update)
-        self.m.add_observer(self.m.message_updated, self.updated)
+                            self.about_to_update.emit)
+        self.m.add_observer(self.m.message_updated, self.updated.emit)
 
-        self.connect(self, SIGNAL('itemSelectionChanged()'),
-                     self.show_selection)
-
-        self.connect(self, SIGNAL('itemDoubleClicked(QTreeWidgetItem*,int)'),
-                     self.double_clicked)
-
-        self.connect(self, SIGNAL('itemCollapsed(QTreeWidgetItem*)'),
-                     lambda x: self.update_column_widths())
-
-        self.connect(self, SIGNAL('itemExpanded(QTreeWidgetItem*)'),
-                     lambda x: self.update_column_widths())
+        self.itemSelectionChanged.connect(self.show_selection)
+        self.itemDoubleClicked.connect(self.double_clicked)
+        self.itemCollapsed.connect(lambda x: self.update_column_widths())
+        self.itemExpanded.connect(lambda x: self.update_column_widths())
 
     def add_toplevel_item(self, txt, icon, hide=False):
         font = self.font()
@@ -222,14 +222,14 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         else:
             font.setItalic(True)
 
-        item = QtGui.QTreeWidgetItem(self)
+        item = QtWidgets.QTreeWidgetItem(self)
         item.setFont(0, font)
         item.setText(0, txt)
         item.setIcon(0, icon)
         if prefs.bold_headers():
             item.setBackground(0, self.palette().midlight())
         if hide:
-            self.setItemHidden(item, True)
+            item.setHidden(True)
 
     def restore_selection(self):
         if not self.old_selection or not self.old_contents:
@@ -241,10 +241,10 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         def mkselect(lst, widget_getter):
             def select(item, current=False):
                 idx = lst.index(item)
-                widget = widget_getter(idx)
+                item = widget_getter(idx)
                 if current:
-                    self.setCurrentItem(widget)
-                self.setItemSelected(widget, True)
+                    self.setCurrentItem(item)
+                item.setSelected(True)
             return select
 
         select_staged = mkselect(new_c.staged, self.staged_item)
@@ -253,17 +253,17 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         select_untracked = mkselect(new_c.untracked, self.untracked_item)
 
         saved_selection = [
-        (set(new_c.staged), old_c.staged, set(old_s.staged),
-            select_staged),
+            (set(new_c.staged), old_c.staged, set(old_s.staged),
+             select_staged),
 
-        (set(new_c.unmerged), old_c.unmerged, set(old_s.unmerged),
-            select_unmerged),
+            (set(new_c.unmerged), old_c.unmerged, set(old_s.unmerged),
+             select_unmerged),
 
-        (set(new_c.modified), old_c.modified, set(old_s.modified),
-            select_modified),
+            (set(new_c.modified), old_c.modified, set(old_s.modified),
+             select_modified),
 
-        (set(new_c.untracked), old_c.untracked, set(old_s.untracked),
-            select_untracked),
+            (set(new_c.untracked), old_c.untracked, set(old_s.untracked),
+             select_untracked),
         ]
 
         # Restore the current item
@@ -272,8 +272,11 @@ class StatusTreeWidget(QtGui.QTreeWidget):
             if category == self.idx_header:
                 item = self.invisibleRootItem().child(idx)
                 if item is not None:
+                    self.blockSignals(True)
                     self.setCurrentItem(item)
-                    self.setItemSelected(item, True)
+                    item.setSelected(True)
+                    self.blockSignals(False)
+                    self.show_selection()
                 return
             # Reselect the current item
             selection_info = saved_selection[category]
@@ -364,9 +367,6 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         parent = self.topLevelItem(idx)
         return parent.child(itemidx)
 
-    def about_to_update(self):
-        self.emit(SIGNAL('about_to_update()'))
-
     def _about_to_update(self):
         self.save_scrollbars()
         self.save_selection()
@@ -400,11 +400,7 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         self.old_selection = self.selection()
         self.old_current_item = self.current_item()
 
-    def updated(self):
-        """Update display from model data."""
-        self.emit(SIGNAL('updated()'))
-
-    def _updated(self):
+    def refresh(self):
         self.set_staged(self.m.staged)
         self.set_modified(self.m.modified)
         self.set_unmerged(self.m.unmerged)
@@ -445,10 +441,8 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         """Add a list of items to a treewidget item."""
         self.blockSignals(True)
         parent = self.topLevelItem(idx)
-        if items:
-            self.setItemHidden(parent, False)
-        else:
-            self.setItemHidden(parent, True)
+        hide = not bool(items)
+        parent.setHidden(hide)
 
         # sip v4.14.7 and below leak memory in parent.takeChildren()
         # so we use this backwards-compatible construct instead
@@ -492,7 +486,8 @@ class StatusTreeWidget(QtGui.QTreeWidget):
     def create_context_menu(self):
         """Set up the status menu for the repo status tree."""
         s = self.selection()
-        menu = QtGui.QMenu(self)
+        menu = qtutils.create_menu('Status', self)
+        menu.addAction(self.launch_editor_action)
 
         selected_indexes = self.selected_indexes()
         if selected_indexes:
@@ -502,12 +497,20 @@ class StatusTreeWidget(QtGui.QTreeWidget):
                 return self._create_header_context_menu(menu, idx)
 
         if s.staged:
-            return self._create_staged_context_menu(menu, s)
+            self._create_staged_context_menu(menu, s)
 
         elif s.unmerged:
-            return self._create_unmerged_context_menu(menu, s)
+            self._create_unmerged_context_menu(menu, s)
         else:
-            return self._create_unstaged_context_menu(menu, s)
+            self._create_unstaged_context_menu(menu, s)
+
+        if not utils.is_win32():
+            menu.addSeparator()
+            menu.addAction(self.default_app_action)
+            menu.addAction(self.parent_dir_action)
+            menu.addAction(self.terminal_action)
+
+        return menu
 
     def _create_header_context_menu(self, menu, idx):
         if idx == self.idx_staged:
@@ -541,33 +544,20 @@ class StatusTreeWidget(QtGui.QTreeWidget):
             action.setShortcut(hotkeys.STAGE_SELECTION)
 
         # Do all of the selected items exist?
-        all_exist = all(not i in self.m.staged_deleted and core.exists(i)
+        all_exist = all(i not in self.m.staged_deleted and core.exists(i)
                         for i in self.staged())
 
         if all_exist:
-            menu.addAction(self.launch_editor_action)
             menu.addAction(self.launch_difftool_action)
 
-        if all_exist and not utils.is_win32():
-            menu.addSeparator()
-            open_default = cmds.run(cmds.OpenDefaultApp, self.staged())
-            action = menu.addAction(icons.default_app(),
-                                    cmds.OpenDefaultApp.name(), open_default)
-            action.setShortcut(hotkeys.PRIMARY_ACTION)
-
-            action = menu.addAction(icons.folder(),
-                                    cmds.OpenParentDir.name(),
-                                    self._open_parent_dir)
-            action.setShortcut(hotkeys.SECONDARY_ACTION)
-
         if self.m.undoable():
-            menu.addSeparator()
             menu.addAction(self.revert_unstaged_edits_action)
 
         menu.addSeparator()
         menu.addAction(self.copy_path_action)
         menu.addAction(self.copy_relpath_action)
         menu.addAction(self.view_history_action)
+        menu.addAction(self.view_blame_action)
         return menu
 
     def _create_staged_submodule_context_menu(self, menu, s):
@@ -575,9 +565,6 @@ class StatusTreeWidget(QtGui.QTreeWidget):
                        cmds.run(cmds.OpenRepo,
                                 core.abspath(s.staged[0])))
 
-        menu.addAction(self.launch_editor_action)
-        menu.addSeparator()
-
         action = menu.addAction(icons.remove(), N_('Unstage Selected'),
                                 cmds.run(cmds.Unstage, self.staged()))
         action.setShortcut(hotkeys.STAGE_SELECTION)
@@ -594,25 +581,12 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         action = menu.addAction(icons.add(), N_('Stage Selected'),
                                 cmds.run(cmds.Stage, self.unstaged()))
         action.setShortcut(hotkeys.STAGE_SELECTION)
-        menu.addSeparator()
-        menu.addAction(self.launch_editor_action)
-
-        if not utils.is_win32():
-            menu.addSeparator()
-            open_default = cmds.run(cmds.OpenDefaultApp, self.unmerged())
-            action = menu.addAction(icons.default_app(),
-                                    cmds.OpenDefaultApp.name(), open_default)
-            action.setShortcut(hotkeys.PRIMARY_ACTION)
-
-            action = menu.addAction(icons.folder(),
-                                    cmds.OpenParentDir.name(),
-                                    self._open_parent_dir)
-            action.setShortcut(hotkeys.SECONDARY_ACTION)
 
         menu.addSeparator()
         menu.addAction(self.copy_path_action)
         menu.addAction(self.copy_relpath_action)
         menu.addAction(self.view_history_action)
+        menu.addAction(self.view_blame_action)
         return menu
 
     def _create_unstaged_context_menu(self, menu, s):
@@ -627,12 +601,9 @@ class StatusTreeWidget(QtGui.QTreeWidget):
             action.setShortcut(hotkeys.STAGE_SELECTION)
 
         # Do all of the selected items exist?
-        all_exist = all(not i in self.m.unstaged_deleted and core.exists(i)
+        all_exist = all(i not in self.m.unstaged_deleted and core.exists(i)
                         for i in self.staged())
 
-        if all_exist and self.unstaged():
-            menu.addAction(self.launch_editor_action)
-
         if all_exist and s.modified and self.m.stageable():
             menu.addAction(self.launch_difftool_action)
 
@@ -641,18 +612,6 @@ class StatusTreeWidget(QtGui.QTreeWidget):
                 menu.addSeparator()
                 menu.addAction(self.revert_unstaged_edits_action)
 
-        if all_exist and self.unstaged() and not utils.is_win32():
-            menu.addSeparator()
-            open_default = cmds.run(cmds.OpenDefaultApp, self.unstaged())
-            action = menu.addAction(icons.default_app(),
-                                    cmds.OpenDefaultApp.name(), open_default)
-            action.setShortcut(hotkeys.PRIMARY_ACTION)
-
-            action = menu.addAction(icons.folder(),
-                                    cmds.OpenParentDir.name(),
-                                    self._open_parent_dir)
-            action.setShortcut(hotkeys.SECONDARY_ACTION)
-
         if all_exist and s.untracked:
             menu.addSeparator()
             if self.move_to_trash_action is not None:
@@ -660,22 +619,20 @@ class StatusTreeWidget(QtGui.QTreeWidget):
             menu.addAction(self.delete_untracked_files_action)
             menu.addSeparator()
             menu.addAction(icons.edit(),
-                           N_('Add to .gitignore'),
-                           cmds.run(cmds.Ignore,
-                                    map(lambda x: '/' + x, self.untracked())))
+                            N_('Add to .gitignore'),
+                            gitignore.gitignore_view)
         menu.addSeparator()
         menu.addAction(self.copy_path_action)
         menu.addAction(self.copy_relpath_action)
         if not selection.selection_model().is_empty():
             menu.addAction(self.view_history_action)
+            menu.addAction(self.view_blame_action)
         return menu
 
     def _create_modified_submodule_context_menu(self, menu, s):
         menu.addAction(icons.cola(), N_('Launch git-cola'),
                        cmds.run(cmds.OpenRepo, core.abspath(s.modified[0])))
 
-        menu.addAction(self.launch_editor_action)
-
         if self.m.stageable():
             menu.addSeparator()
             action = menu.addAction(icons.add(), N_('Stage Selected'),
@@ -688,7 +645,6 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         menu.addAction(self.view_history_action)
         return menu
 
-
     def _delete_untracked_files(self):
         cmds.do(cmds.Delete, self.untracked())
 
@@ -785,7 +741,7 @@ class StatusTreeWidget(QtGui.QTreeWidget):
             hscrollbar.setValue(hscroll)
 
         self.setCurrentItem(item)
-        self.setItemSelected(item, True)
+        item.setSelected(True)
 
     def staged(self):
         return self._subtree_selection(self.idx_staged, self.m.staged)
@@ -830,12 +786,6 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         """Called when an item is double-clicked in the repo status tree."""
         cmds.do(cmds.StageOrUnstage)
 
-    def _open_using_default_app(self):
-        cmds.do(cmds.OpenDefaultApp, self.selected_group())
-
-    def _open_parent_dir(self):
-        cmds.do(cmds.OpenParentDir, self.selected_group())
-
     def show_selection(self):
         """Show the selected item."""
         # Sync the selection model
@@ -857,7 +807,7 @@ class StatusTreeWidget(QtGui.QTreeWidget):
                 self.idx_staged: cmds.DiffStagedSummary,
                 self.idx_modified: cmds.Diffstat,
                 # TODO implement UnmergedSummary
-                #self.idx_unmerged: cmds.UnmergedSummary,
+                # self.idx_unmerged: cmds.UnmergedSummary,
                 self.idx_untracked: cmds.UntrackedSummary,
             }.get(idx, cmds.Diffstat)
             cmds.do(cls)
@@ -928,40 +878,47 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         """Copy a selected relative path to the clipboard"""
         self.copy_path(absolute=False)
 
+    def _item_filter(self, item):
+        return not item.deleted and core.exists(item.path)
+
     def mimeData(self, items):
         """Return a list of absolute-path URLs"""
-        item_filter = lambda item: not item.deleted and core.exists(item.path)
-        paths = qtutils.paths_from_items(items, item_filter=item_filter)
+        paths = qtutils.paths_from_items(items, item_filter=self._item_filter)
         return qtutils.mimedata_from_paths(paths)
 
     def mimeTypes(self):
         return qtutils.path_mimetypes()
 
+    def view_blame(self):
+        """Signal that we should view blame for paths."""
+        cmds.do(cmds.BlamePaths, selection.union(selection.selection_model()))
+
     def view_history(self):
         """Signal that we should view history for paths."""
-        cmds.do(cmds.VisualizePaths, selection.union(selection.selection_model()))
+        cmds.do(cmds.VisualizePaths,
+                selection.union(selection.selection_model()))
 
 
-class StatusFilterWidget(QtGui.QWidget):
+class StatusFilterWidget(QtWidgets.QWidget):
 
     def __init__(self, parent=None):
-        QtGui.QWidget.__init__(self, parent)
+        QtWidgets.QWidget.__init__(self, parent)
         self.main_model = main.model()
 
         hint = N_('Filter paths...')
         self.text = completion.GitStatusFilterLineEdit(hint=hint, parent=self)
         self.text.setToolTip(hint)
-        self.text.hint.enable(True)
         self.setFocusProxy(self.text)
         self._filter = None
 
         self.main_layout = qtutils.hbox(defs.no_margin, defs.spacing, self.text)
         self.setLayout(self.main_layout)
 
-        self.connect(self.text, SIGNAL('changed()'), self.apply_filter)
-        self.connect(self.text, SIGNAL('cleared()'), self.apply_filter)
-        self.connect(self.text, SIGNAL('return()'), self.apply_filter)
-        self.connect(self.text, SIGNAL('editingFinished()'), self.apply_filter)
+        text = self.text
+        text.changed.connect(self.apply_filter)
+        text.cleared.connect(self.apply_filter)
+        text.enter.connect(self.apply_filter)
+        text.editingFinished.connect(self.apply_filter)
 
     def apply_filter(self):
         text = self.text.value()
